import QtQuick 1.1
import Common 1.0

// Apply object flying effect. Must be placed on game level over belt pack.
ObjBase {
    property QtObject targetItem
    property real targetX
    property real targetY
    property real scaleFrom
    property real scaleTo
    property int duration: 700

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    function start(item, x, y, duration_, scaleFrom_, scaleTo_) {
        animation.complete()
        targetItem = item
        img.x = x
        img.y = y
        var p = item.mapToItem(game, 0, 0)
        targetX = p.x
        targetY = p.y
        duration = duration_
                || Math.max(200, 0.7 * Math.sqrt((x - p.x) * (x - p.x) + (y - p.y) * (y - p.y)))
        scaleFrom = scaleFrom_ || 1
        scaleTo = scaleTo_ || 0.7
        animation.start()
    }

    id: self
    visible: targetItem != null
    hoverEnabled: true
    anchors.fill: parent

    onPressed: animation.complete()

    SequentialAnimation {
        id: animation
        PropertyAction { target: targetItem; property: "visible"; value: false }

        ParallelAnimation {
            NumberAnimation { target: img; property: "rotation"; duration: self.duration; from: 360; to: 0 }
            NumberAnimation { target: img; property: "x"; duration: self.duration; to: targetX }
            NumberAnimation { target: img; property: "y"; duration: self.duration; to: targetY }
            NumberAnimation { target: img; property: "scale"; duration: self.duration; from: scaleFrom; to: scaleTo }
        }

        PropertyAction { target: targetItem; property: "visible"; value: true }
        ScriptAction { script: translateCorrection.x = translateCorrection.y = 0 }
        ScriptAction { script: targetItem = null }
    }

    // Correct target posistion while flying
    Timer {
        repeat: true
        running: targetItem != null
        interval: 50
        onTriggered: {
            var p = targetItem.mapToItem(game, 0, 0)
            translateCorrection.x = p.x - targetX
            translateCorrection.y = p.y - targetY
        }
    }

    QtQuickImage {
        id: img
        asynchronous: true
        source: targetItem ? targetItem.imageSource : ""

        transform: Translate {
            id: translateCorrection
            Behavior on x { NumberAnimation { duration: 250 } }
            Behavior on y { NumberAnimation { duration: 250 } }
        }
    }
}
