import QtQuick 1.1
import Common 1.0

// Plays ActionSound from a pool.
// ActionSoundPool is suitable when sound is longer than period for sound starting (for example time between clicks).
ObjectContainer {

    property bool playing: numPlaying > 0
    property int numPlaying // Set automatically
    property bool neverStop: true

    default property Component template // ActionSound template for pool item
    property int maxSounds: 3 // Pool size

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: self

    onPlayingChanged: {
        if (playing) {
            if (numPlaying == 0 ) {
                play()
            }
        } else if (!neverStop) {
            fadeOut()
        }
    }

    function play() {
        for (var i = 0; i < count(); ++i) {
            if (!at(i)._playing() && at(i).toggleCountdown <= 0) {
                at(i).play()
                break
            }
        }
    }

    function fadeOut() {
        for (var i = 0; i < count(); ++i) {
            at(i).fadeOut()
        }
    }

    function _checkPlaying() {
        var p = 0
        for (var i = 0; i < count(); ++i) {
            if (at(i)._playing() || at(i).toggleCountdown > 0) {
                ++p
            }
        }
        numPlaying = p
    }

    Component.onCompleted: {
        for (var i = 0; i < maxSounds; ++i) {
            var o = template.createObject(self)
            o.group = self
            append(o)
        }
    }
}
