// Convenient component to declare an ambient sound.
VariateAudioSource {
    // If minPauseTime < maxPauseTime the sound will be played and played again with repeats paused
    // in range [minPauseTime, maxPauseTime) randomly.
    // This is a convenient feature for environmental audio effects. Time is in milliseconds.
    property int minPauseTime: 5000
    property int maxPauseTime: 20000

    // Set by the parent ScreenAudio
    property ScreenAudio screenAudio

    category: "Ambient"
    fadeInTime: 0 // No fading by default
    reverberationEnabled: true

    onPlayingChanged: playDelayed()

    function playDelayed() {
        if (!playing && toggleCountdown <= 0 && minPauseTime < maxPauseTime && minPauseTime >= 0
                && screen.state === "") {
            variateParams()
            if (screenAudio.canPlay()) {
                toggleCountdown = minPauseTime + Math.random() * (maxPauseTime - minPauseTime)
                                + 1 // toggleCountdown must be > 0
            } else {
                console.log("Too much ambient sounds")
                toggleCountdown = 30000 // penalty
            }
        }
    }
}
