import QtQuick 1.1
import Common 1.0
import Audio 1.0

// Container for game audio
ObjectContainer {

    // The property is set automatically
    property Music currentMusic

    // The property is null if not playing. It is set automatically.
    property DynamicAudioSource awardMusicSource

    // If the player is a first time in a location the property below is used
    property int locationObservationMusicDelay: 10000 // in milliseconds

    property bool musicEnabled: true

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading


    onMusicEnabledChanged: {
        if (currentMusic) {
            if (musicEnabled) {
                currentMusic.beginPlay()
            } else {
                currentMusic.stop()
                audio.stopAwardMusic()
            }
        }
    }


    // Plays award music stopping the current one.
    // If the same track is already playing just leave it.
    // Gain and delay parameters can be omitted.
    function playAwardMusic(name, gain, delay, fadeInTime) {
        if (currentMusic) {
            currentMusic.stop()
        }
        stopAwardMusic()

        awardMusicSource = audioEngine.createOrGetDynamicAudioSource("Music", name)
        awardMusicSource.gain = gain || 1
        awardMusicSource.source = Utils.resolvedUrl(root, "Audio/Music/" + name + ".ogg")
        if (awardMusicSource.playing) {
            awardMusicSource.fadeInTime = awardMusicSource.fadeOutTime
            awardMusicSource.fadeIn() // Back fading
        } else {
            awardMusicSource.fadeInTime = fadeInTime || 1
            awardMusicSource.toggleCountdown = delay || 1
        }
    }

    function stopAwardMusic() {
        if (awardMusicSource) {
            awardMusicSource.toggleCountdown = 0
            awardMusicSource.fadeOut()
            awardMusicSource = null
        }
    }

    onAwardMusicSourceChanged: {
        if (!awardMusicSource && currentMusic) {
            currentMusic.beginPlay()
        }
    }

    Connections {
        target: awardMusicSource
        onPlayingChanged: if (!awardMusicSource.playing) awardMusicSource = null
    }
}
