import QtQuick 1.1
import Common 1.0

// Declares a music sequence
ObjectContainer {

    property int pauseBetweenTracksTime: 5000 // in milliseconds
    property int fadeInTime: 700
    property int fadeOutTime: 700
    property bool sequencial: true
    property bool playFromOffset: true

    // The properties below are not set by the user
    property int currentTrackIndex: -1
    property bool enabled: false

    id: self

    // The function is called onScreenLoaded
    function beginPlay() {
        //console.debug("Music.beginPlay()")
        if (!enabled) {
            enabled = true

            // Find the current playing music audio source
            var currentFadingOutMusic
            for (var i = 0; i < audioEngine.dynamicSources.length; ++i) {
                var src = audioEngine.dynamicSources[i]
                if (src.category === "Music") {
                    currentFadingOutMusic = src
                    break
                }
            }

            // Try to proceed playing of the same music
            if (currentFadingOutMusic) {
                for (i = 0; i < data.length; ++i) {
                    if (data[i].name === currentFadingOutMusic.name) {
                        currentTrackIndex = i
                        //console.debug("Music.beginPlay(): proceed the music track #", currentTrackIndex)
                        data[i].play(self, 1, fadeInTime)
                        return
                    }
                }
            }

            var delay = activeLocation && activeLocation.firstTimeInLocation
                    ? locationObservationMusicDelay : 1
            playNext(delay, playFromOffset)
        }
    }

    // Set delay to 1 if you want to play immediately
    function playNext(delay, fromStopOffset) {
        if (enabled && data.length > 0) {
            if (sequencial) {
                ++currentTrackIndex
            } else {
                var i = Math.round(Math.random() * data.length)
                currentTrackIndex = currentTrackIndex != i ? i : i + 1
            }
            if (currentTrackIndex >= data.length) {
                currentTrackIndex = 0
            }
            //console.debug("Music.playNext(): #", currentTrackIndex)
            data[currentTrackIndex].play(self, delay || pauseBetweenTracksTime, fadeInTime, fromStopOffset)
        }
    }

    function stop(delay) {
        //console.debug("Music.stop(): #", currentTrackIndex)
        enabled = false
        if (0 <= currentTrackIndex && currentTrackIndex < data.length) {
            var track = data[currentTrackIndex]
            if (track.audioSource) {
                track.audioSource.fadeOutTime = fadeOutTime
                if (delay) {
                    if (track.audioSource.playing) {
                        track.audioSource.toggleCountdown = delay
                    }
                } else {
                    // Stop immediately - actual for screen change
                    track.audioSource.toggleCountdown = 0
                    track.audioSource.fadeOut()
                }
            }
        }
    }
}
