import Audio 1.0
import QtQuick 1.1

// Convenient component to declare a music track. That should be use together with Music component.
QtObject {

    property string name
    property real gain: 1

    // audioSource is null if not playing. The property is set automatically.
    property DynamicAudioSource audioSource
    // Reference to the parent container. The property is set automatically.
    property Music music
    // Readonly file name
    property url source: Utils.resolvedUrl(root, "Audio/Music/" + name + ".ogg")
    // Time when the track stopped
    property int stopOffset: audioSource && audioSource.stopOffset > 0 ? audioSource.stopOffset : stopOffset

    id: self

    onAudioSourceChanged: {
        if (!audioSource) {
            //console.debug("MusicTrack.onAudioSourceChanged: music.playNext()")
            music.playNext()
        }
    }

    function play(music, delay, fadeInTime, fromStopOffset) {
        self.music = music
        audioSource = audioEngine.createOrGetDynamicAudioSource("Music", name)
        audioSource.gain = gain
        audioSource.source = source
        if (audioSource.playing) {
            audioSource.fadeInTime = audioSource.fadeOutTime
            audioSource.fadeIn() // Back fading
        } else {
            audioSource.fadeInTime = fadeInTime || 1
            if (fromStopOffset && stopOffset) {
                audioSource.fadeIn(stopOffset)
            } else {
                audioSource.toggleCountdown = delay || 1
            }
        }
    }

    property Connections _connectionsWithAudioSource: Connections {
        target: audioSource
        onPlayingChanged: if (!audioSource.playing) audioSource = null
    }
}
