import QtQuick 1.1
import Common 1.0
import Audio 1.0

// Declares ambient sounds and music for the screen
ObjectContainer {
    // Reference to the music group
    property QtObject music
    // Limits simultaneously playing sounds
    property int maxSounds: 16
    // One of Reverberation.Preset enum values
    //    Default,
    //    Generic,
    //    PaddedCell,
    //    Room,
    //    Bathroom,
    //    LivingRoom,
    //    StoneRoom,
    //    Auditorium,
    //    ConcertHall,
    //    Cave,
    //    Arena,
    //    Hangar,
    //    CarpetedHallway,
    //    Hallway,
    //    StoneCorridor,
    //    Alley,
    //    Forest,
    //    City,
    //    Mountains,
    //    Quarry,
    //    Plain,
    //    ParkingLot,
    //    SewerPipe,
    //    Underwater
    property int reverberationPreset: Reverberation.Default

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: screenAudio

    property Connections _screenSignals: Connections {
        target: screen
        onScreenLoaded: {
            for (var i = 0; i < data.length; ++i) {
                var snd = data[i]
                snd.screenAudio = screenAudio
                snd.playDelayed()
            }

            if (music) {
                audio.currentMusic = music
                if (audio.musicEnabled) {
                    music.beginPlay()
                }
            }
            audioEngine.reverberation.preset = reverberationPreset
        }
        onScreenLeft: {
            for (var i = 0; i < data.length; ++i) {
                var snd = data[i]
                snd.toggleCountdown = 0
                snd.fadeOut()
            }

            if (music) {
                music.stop()
                audio.currentMusic = null
            }
            audio.stopAwardMusic()
            audioEngine.reverberation.preset = Reverberation.Default
        }
    }

    // Returns true if number of playing in the current moment sounds is less than maxCount
    function canPlay() {
        var playCount = 0
        for (var i = 0; i < data.length; ++i) {
            playCount += data[i].playing
        }
        return playCount < maxSounds
    }
}
