import Audio 1.0

// Convenient component to control ambient sounds and ambient music.
// The derived components MusicTrack and AmbientSound are more suitable for most cases.
AudioSource {

    // Audio file name without extension and folder.
    // Folder is selected basing on the audio category, for example: Music, Ambient.
    property string name

    // If minGain < maxGain gain will be set randomly in range [minGain, maxGain).
    // If the sound has no distance attenuation the max possible gain is 1.
    property real minGain: 1
    property real maxGain: 1

    // If minPitch < maxPitch pitch will be set randomly in range [minPitch, maxPitch)
    property real minPitch: 1
    property real maxPitch: 1

    // If minPanning < maxPanning panning will be set randomly in range [minPanning, maxPanning]
    property real minPanning: 0
    property real maxPanning: 0

    // Range is [-1, 1] - from left to right.
    property real panning

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    /*
      Other useful properties from base AudioSource component:
        position - works for mono audio only
        velocity
        gain - 1 by default
        pitch - 1 by default
        source (fileName) - readonly
        streaming - AudioSource.StreamingAuto by default, can be also StreamingNo or StreamingYes
        preloaded - if true the sound is always in memory, false by default
        playing - readonly
        buffer - readonly, access to channels, frequency, actualStreaming
        looping
        fadeInTime
        fadeOutTime
        category -  controls the folder name and useful for category gain setting.
      */

    engine: audioEngine
    source: Utils.resolvedUrl(root, "Audio/" + category + "/" + name + ".ogg")
    position: Qt.vector3d(panning, 0, 1)

    function variateParams() {
        if (minGain < maxGain) {
            gain = minGain + Math.random() * (maxGain - minGain)
        }
        if (minPitch < maxPitch) {
            pitch = minPitch + Math.random() * (maxPitch - minPitch)
        }
        if (minPanning < maxPanning) {
            panning = minPanning + Math.random() * (maxPanning - minPanning)
        }
    }
}
