import QtQuick 1.1
import Common 1.0


// Zoom zone object that can be collected in the belt pack.
// BeltZoomZone will be removed from the belt back in state "Done"
ZoomZone {

    // The index below is optionally filled by the belt pack
    property int beltPackIndex
    // Internal property to correctly load game state
    property int _beltPackItemIndex

    id: self
    state: "ReadyToCollect" // When state == "Closed" the object is in the belt pack
    hintable: state === "ReadyToCollect"

    // Puts the object to the belt pack if it's possible
    function putToBeltPack() {
        if (state == "ReadyToCollect") {
            state = "Closed"
            beltPack.itemAdded(self)
            // BeltZoomZone from another screen or location can be put to the belt pack, so, load its images
            Utils.setInactive(self, false)
        }
    }

    onClicked: putToBeltPack()

    // We need to notify binding when beltback transformation or visibility changed
    Connections {
        target: visible ? beltPack : null
        onMovementEnded: if (container) container.zoomZoneChanged() // A way to notify binding
        onVisibleChanged: if (container) container.zoomZoneChanged() // A way to notify binding
    }

    // Independent state group allows derived components to utilize build-in state properties.
    StateGroup {
        state: self.state
        states: [
            State {
                name: "Closed"
                StateChangeScript { script: if (parent !== beltPack.container) self.x = self.y = 0 }
                PropertyChanges { target: self; parent: beltPack.container; hoverEnabled: true }
            },
            State {
                name: "Zoomed"
                PropertyChanges { target: self; parent: beltPack.container }
            },
            State {
                name: "Empty"; extend: "Done"
            },
            State {
                name: "Done"
                PropertyChanges {
                    target: self; visible: false; cx: game.width * 0.5; cy: game.height * 0.5
                }
                StateChangeScript {
                    script: { beltPack.itemRemoved(self); Utils.setInactive(self, true) }
                }
            }
        ]
    }

    // Save/load support
    function saveState() {
        var data = state ? {state: state} : {}
        if (_beltPackItemIndex) {
            data.beltPackItemIndex = _beltPackItemIndex
        }
        return data
    }
    function loadState(data) {
        _beltPackItemIndex = data.beltPackItemIndex || 0
        if (data.state === "Closed" || data.state === "Zoomed") {
            state = "ReadyToCollect"
            beltPack.loadedObjects.setAt(_beltPackItemIndex, self)
        } else {
            state = data.state || ""
        }
    }
}
