import QtQuick 1.1
import Common 1.0


// Manages conversations.
// It displays one cue in a moment and changes to the next one following to the user input
// and basing on the conversation structure.
ObjBase {

    // Points the current open cue. Readonly.
    property QtObject activeCue
    // Fired when the conversation is finalized.
    signal finalized

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: self
    anchors.fill: parent
    visible: false
    z: 10000

    onVisibleChanged: game.conversationVisibilityChanged(self)

    onClicked: onClickedImpl()
    
    function onClickedImpl(){
          if (activeCue) self.nextCue() 
    }

    Loader {
        sourceComponent: activeCue ? activeCue.template : null
        onItemChanged: if (item) Utils.setInactive(item, false)
    }


    // Starts the conversation.
    function open() {
        visible = true
        if (!activeCue) {
            for (var i = 0; i < resources.length; ++i) {
                if (resources[i].template !== undefined) { // Check instance of Cue
                    nextCue(resources[i])
                    break
                }
            }
        }
    }

    // Stops the conversation.
    function close(resetActiveCue) {
        visible = false
        if (resetActiveCue) {
            activeCue = null
        }
    }

    // Switches to the next cue
    function nextCue(next) {
        if (!next && activeCue) {
            next = activeCue.next
            // Next is the next sibling by default
            if (!next) {
                for (var i = 0; i < resources.length - 1; ++i) {
                    if (activeCue === resources[i]) {
                        next = resources[i + 1]
                        break
                    }
                }
            }
        }
        if (next && (!activeCue || !activeCue.finale)) {
            activeCue = next
        } else {
            close(true)
            finalized()
        }
    }
}
