import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0

// Cover made of mask and background image. Convenient way to represent layers in game.
ShaderEffectItem {

    property Image mask: Image {}
    property Image background: Image { source: Utils.resolvedUrl(self, "Back.png") }
    property variant bgScale: Qt.point(mask.width / background.width, mask.height / background.height)
    property variant bgOffset: Qt.point(self.x / background.width, self.y / background.height)
    property variant maskSampler: mask
    property variant backgroundSampler: background

    id: self
    width: mask.width
    height: mask.height
    blending: true
    shaderProgramName: "Cover" // Name to share program between ShaderEffectItem instances

    vertexShader: "
        uniform mediump mat4 qt_ModelViewProjectionMatrix;
        attribute mediump vec4 qt_Vertex;
        attribute mediump vec2 qt_MultiTexCoord0;
        varying mediump vec2 backTexCoord;
        varying mediump vec2 maskTexCoord;
        uniform mediump vec2 bgScale;
        uniform mediump vec2 bgOffset;
        void main(void)
        {
            backTexCoord = bgOffset + qt_MultiTexCoord0 * bgScale;
            maskTexCoord = qt_MultiTexCoord0;
            gl_Position = qt_ModelViewProjectionMatrix * qt_Vertex;
        }"

    fragmentShader: "
        varying mediump vec2 backTexCoord;
        varying mediump vec2 maskTexCoord;
        uniform lowp sampler2D maskSampler;
        uniform lowp sampler2D backgroundSampler;
        uniform lowp float qt_Opacity;
        void main(void)
        {
            lowp vec4 b = texture2D(backgroundSampler, backTexCoord);
            lowp vec4 m = texture2D(maskSampler, maskTexCoord);
            gl_FragColor =  b * (m.a * qt_Opacity);
        }"
}
