import QtQuick 1.1
import Common 1.0

// FIXME: ApplyObjList does not work
ObjBase { // Reset cursor with ObjBase
    anchors.fill: parent

    Rectangle {
        anchors.left: parent.left; anchors.right: parent.right
        height: childrenRect.height + 9
        opacity: 0.85

        ListModel { id: objects }

        Text {
            id: countText
            y: 6; height: 28; font.pixelSize: 17
            anchors.horizontalCenter: parent.horizontalCenter
            text: "Found apply objects: " + objects.count
            color: "DarkCyan"
        }

        Flow {
            anchors.top: countText.bottom
            anchors.left: parent.left; anchors.right: parent.right

            Repeater {
                model: objects
                Rectangle {
                    color: "#f4f4f4"
                    width: 96; height: 96
                    border.width: 1
                    border.color: "DarkGrey"
                    QtQuickImage {
                        anchors.fill: parent; anchors.margins: 2
                        source: obj.debugImageSource
                        asynchronous: true
                    }
                    Text {
                        anchors.horizontalCenter: parent.horizontalCenter;
                        anchors.bottom: parent.bottom; anchors.bottomMargin: 2
                        text: name; font.pixelSize: 8
                    }
                    MouseArea {
                        anchors.fill: parent
                        onClicked: obj.putToBeltPack()
                    }
                }
            }
        }

        onVisibleChanged: fillModel()
        Connections {
            target: game
            onLocationChanged: fillModel()
        }
        Connections {
            target: beltPack
            onItemAdded: fillModel()
        }
    }


    // Fills objects ListModel by ApplyObj-s from the visible location
    function fillModel() {
        if (visible) {
            objects.clear()
            for (var i = 0; i < game.children.length; ++i) {
                var item = game.children[i]
                if (item.visible && item.opacity) {
                    fillModelRecursively(item, "game")
                }
            }
        }
    }

    function fillModelRecursively(item, parentName) {
        var qmlObjId = Utils.qmlObjId(item)
        if (item.beltPackCollectable
                && item.state !== "Applied" && item.state !== "InBeltPack") {

            objects.append({obj: item, name: parentName + "/" + qmlObjId})
        }
        for (var i = 0; i < item.children.length; ++i) {
            fillModelRecursively(item.children[i], item.onScreenLoaded ? qmlObjId : parentName)
        }
    }
}
