import QtQuick 1.1
import Common 1.0
import Qt.labs.canvas 1.0


// Draws rectangle area, also supplies convenient converters
Canvas {

    property string offset: "0, 0"
    property string pointsCode

    opacity: 0.5
    anchors.fill: parent
    color: Qt.rgba(0, 0, 0, 0)

    property QtObject itemObj

    onPointsCodeChanged: if (visible) requestPaintTimer.start()
    onVisibleChanged: if (visible) requestPaintTimer.start()

    function parseCodeAndPaint(pointsCode) {
        if (visible) {
            try {
                itemObj = Qt.createQmlObject("import QtQuick 1.1\n import Common 1.0\n Item { visible: false; " + pointsCode + "}",
                                           self, "dynamicPath")
            }
            catch (e) {
                console.log("Syntax error on code parsing")
            }
        }
    }

    Timer {
        id: requestPaintTimer; interval: 50; onTriggered: requestPaint()
    }

    onPaint: {
        var context = getContext("2d")
        context.clearRect(0, 0, width, height)
        context.beginPath()
        context.strokeStyle = "Blue"
        context.fillStyle = "Cyan"
        context.lineWidth = 2

        parseCodeAndPaint(pointsCode)
        var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])

        if (itemObj) {
            context.moveTo(itemObj.x + x , itemObj.y + y )
            context.lineTo(itemObj.x + x + itemObj.width, itemObj.y + y )
            context.lineTo(itemObj.x + x + itemObj.width, itemObj.y + y + itemObj.height )
            context.lineTo(itemObj.x + x , itemObj.y + y + itemObj.height )
            context.lineTo(itemObj.x + x , itemObj.y + y )
        }

        context.fill()
        context.stroke()
    }

    function convertPointsToString(offset) {
        var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
        var str = ""
        while (points.count > 2) {
            points.remove(0)
        }
        if (points.count === 1) {
            var s = points.get(0)
            str = "x: " + (s.px - x) + "; y: " + (s.py - y) + "\n"
        }
        if (points.count > 1) {
            var s = points.get(0)
            var e = points.get(1)

            str = "x: " + (Math.min(s.px, e.px) - x) + "; y: " + (Math.min(s.py, e.py) - y) + "\n"
            str += "width: " + (Math.max(s.px, e.px) - Math.min(s.px, e.px)) + "; height: " + (Math.max(s.py, e.py) - Math.min(s.py, e.py)) + "\n"
        }
        return str
    }

    function convertCodeToPoints(pointsCode, offset) {
        if (!dragActive) {
            parseCodeAndPaint(pointsCode)
            var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
            if (isNaN(y) || isNaN(x)) return
            points.clear()
            if (pointsCode) {
                points.append({px: itemObj.x + x  , py: itemObj.y + y})
                points.append({px: itemObj.x + x + itemObj.width  , py: itemObj.y + y + itemObj.height })
            }
        }
    }
}
