import QtQuick 1.1
import Common 1.0
import Qt.labs.canvas 1.0


// Draws points as polygon, also supplies convenient converters
Canvas {

    property string offset: "0, 0"
    property string pointsCode

    opacity: 0.5
    anchors.fill: parent
    color: Qt.rgba(0, 0, 0, 0)

    onPointsCodeChanged: if (visible) requestPaintTimer.start()
    onVisibleChanged: if (visible) requestPaintTimer.start()

    Timer {
        id: requestPaintTimer; interval: 50; onTriggered: requestPaint()
    }

    onPaint: {
        var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
        var context = getContext("2d")
        context.clearRect(0, 0, width, height)
        context.beginPath()
        context.strokeStyle = "Blue"
        context.fillStyle = "Cyan"
        context.lineWidth = 2

        var vertices = pointsCode.split(",")
        if (vertices.length > 0) {
            var px = parseInt(vertices[vertices.length - 2]) + x
            var py = parseInt(vertices[vertices.length - 1]) + y
            context.moveTo(px, py)
            for (var i = 0; i < vertices.length; i += 2) {
                context.lineTo(parseInt(vertices[i]) + x, parseInt(vertices[i + 1]) + y)
            }
        }

        context.fill()
        context.stroke()
    }

    function convertPointsToString(offset) {
        var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
        var str = ""
        for (var i = 0; i < points.count; ++i) {
            var p = points.get(i)
            str += (p.px - x) + "," + (p.py - y)
            if (i !== points.count - 1) {
                str += ", "
            }
        }
        return str
    }

    function convertCodeToPoints(pointsCode, offset) {
        if (!dragActive) {
            var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
            if (isNaN(y) || isNaN(x)) return
            var vertices = pointsCode.split(",")
            points.clear()
            for (var i = 0; i < vertices.length - 1; i += 2) {
                points.append({px: parseInt(vertices[i]) + x,
                               py: parseInt(vertices[i + 1]) + y})
            }
        }
    }
}
