import QtQuick 1.1
import Qt.labs.canvas 1.0


// Draws points as spline, also supplies convenient converters
Canvas {

    property string offset: "0, 0"
    property string pointsCode

    id: self
    opacity: 0.8
    anchors.fill: parent
    color: Qt.rgba(0, 0, 0, 0)

    property Path path

    onPointsCodeChanged: parseCodeAndPaint(pointsCode)
    onVisibleChanged: parseCodeAndPaint(pointsCode)

    function parseCodeAndPaint(pointsCode) {
        if (visible) {
            try {
                path = Qt.createQmlObject("import QtQuick 1.1\n import Common 1.0\n Path {" + pointsCode + "}",
                                           self, "dynamicPath")
                requestPaintTimer.start()
            }
            catch (e) {
                console.log("Syntax error on path parsing")
            }
        }
    }

    Timer {
        id: requestPaintTimer; interval: 100; onTriggered: requestPaint()
    }

    onPaint: {
        var context = getContext("2d")
        context.save()
        context.clearRect(0, 0, width, height)
        context.beginPath()
        context.translate(parseInt(offset.split(",")[0]), parseInt(offset.split(",")[1]))
        context.strokeStyle = "Cyan"
        context.lineWidth = 3
        context.addPath(path)
        context.stroke()
        context.restore()
    }

    function convertPointsToString(offset) {
        var str = ""
        if (points.count > 0) {
            var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
            var p = points.get(0)
            str += "startX: " + (p.px - x) + "; startY: " + (p.py - y) + "\n"
            for (var i = 1; i < points.count; ++i) {
                p = points.get(i)
                str += "PathCurve { x: " + (p.px - x) + "; y: " + (p.py - y) + " }\n"
            }
        }
        return str
    }

    function convertCodeToPoints(pointsCode, offset) {
        if (!dragActive) {
            parseCodeAndPaint(pointsCode)
            var x = parseInt(offset.split(",")[0]), y = parseInt(offset.split(",")[1])
            if (isNaN(y) || isNaN(x)) return
            points.clear()
            if (pointsCode) {
                points.append({px: path.startX + x, py: path.startY + y})
            }
            for (var i = 0; i < path.pathElements.length; ++i) {
                var p = path.pathElements[i]
                points.append({px: p.x + x, py: p.y + y})
            }
        }
    }
}
