import QtQuick 1.1
import Common 1.0

// Edits points for path or polygon. ObjBase resets cursor.
ObjBase {

    property alias points: points
    property bool dragActive: false
    signal pointsCommited(variant points)

    anchors.fill: parent

    ListModel { id: points }

    Repeater {
        anchors.fill: parent
        model: points

        Rectangle {
            width: 10; height: 10
            radius: 5
            border.width: 2
            border.color: mouseArea.containsMouse ? "Red" : "LightSteelBlue"
            color: mouseArea.containsMouse ? "White" : "#f2f2f2"
            opacity: 0.9
            x: px - width / 2
            y: py - height / 2

            MouseArea {
                id: mouseArea
                anchors.fill: parent
                hoverEnabled: true
                drag.target: parent
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                onPositionChanged: {
                    if (pressedButtons & Qt.LeftButton) {
                        dragActive = true
                        points.set(index, {px: parent.x + parent.width / 2, py: parent.y + parent.height / 2})
                        pointsCommited(points)
                    }
                }
                onReleased: {
                    dragActive = false
                    points.set(index, {px: parent.x + parent.width / 2, py: parent.y + parent.height / 2})
                    pointsCommited(points)
                }
                onClicked: {
                    if (mouse.button === Qt.RightButton) {
                        points.remove(index)
                        pointsCommited(points)
                    }
                }
            }
        }
    }

    onClicked: {
        if (mouse.button === Qt.LeftButton) {
            points.append({px: mouse.x, py: mouse.y})
            pointsCommited(points)
        }
    }
}
