import QtQuick 1.1

// Display layering rate
Rectangle {
    property real layering
    property real criticalWeight: Math.max(0, Math.min(layering - 5, 3)) / 3
    property Item layeringRoot: game
    property int precision: 2
    property int updatePeriod: 100

    height: layeringText.height + 2
    width: layeringText.width + 2
    opacity: 0.6 + criticalWeight * 0.4
    color: Qt.rgba(1, 1 - criticalWeight, 1 - criticalWeight, 1)


    Text {
        id: layeringText
        text: layering
        font.pixelSize: 14 + 10 * criticalWeight
        x: 1
    }

    Timer {
        repeat: true; running: true; interval: updatePeriod
        onTriggered: layering = calcLayering()
    }

    function calcLayering() {
        return (calcArea(layeringRoot, 1) / (layeringRoot.width * layeringRoot.height)).toFixed(precision)
    }

    function calcArea(item, parentScale) {
        var scale = parentScale * item.scale
        var area = 0
        if (item.visible && item.opacity && item.scale && !item.skipLayeringCalculation) {

            if ((item.source && item.sourceSize !== undefined) // That must be Image, BorderImage or AnimatedSprite
                    || (item.fragmentShader && item.meshResolution !== undefined) // ShaderEffectItem
                    || item.showSceneGraph !== undefined // Viewport for 3D
                    || (item.color !== undefined && item.gradient !== undefined) // Rectangle
                    || (item.color !== undefined && item.lineHeightMode !== undefined) // Text
                    ) {
                area += item.width * item.height * scale
            }

            if (item.computeParticlesArea) {
                area += item.computeParticlesArea()
            }

            if (item.showSceneGraph === undefined // Do not go in 3D
                    && item.computeParticlesArea === undefined // Do not go in ParticleSystem
                    ) {
                for (var i = 0; i < item.children.length; ++i) {
                    area += calcArea(item.children[i], scale)
                }
            }
        }
        return area
    }
}
