import QtQuick 1.1
import Common 1.0


// Navigation to the selected screen
Rectangle {
    width: 400
    height: 550
    anchors.centerIn: parent
    radius: 4
    border.width: 2

    // Reset cursor with ObjBase on this area
    ObjBase {
        anchors.fill: parent
    }

    // Header
    Rectangle {
        anchors.left: parent.left; anchors.right: parent.right
        height: 34
        color: "DarkBlue"
        y: -32; radius: 4
        Text {
            text: "Navigate to screen"
            color: "White"
            anchors.centerIn: parent
            x: 10
            font.pixelSize: 17
            font.weight: Font.Bold
        }
    }

    Text {
        text: "filter:"
        y: 8
        anchors.right: editBoxFrame.left; anchors.rightMargin: 10
        font.pixelSize: 12
    }

    Rectangle {
        id: editBoxFrame
        y: 4; width: 100; height: 20
        anchors.right: parent.right; anchors.rightMargin: 10
        color: "Transparent"
        border.width: 1

        TextInput {
            id: filterText
            anchors.fill: parent; anchors.leftMargin: 2; anchors.rightMargin: 2
            onTextChanged: model.refill()
        }
    }

    ListModel {
        id: model

        function refill() {
            var ftext = filterText.text.toLowerCase()
            list.currentIndex = 0
            model.clear()
            for (var i = 0; i < locations.count(); ++i) {
                var location = locations.at(i)
                var locationName = Utils.qmlObjId(location)
                for (var j = 0; j < location.screens.length; ++j) {
                    var screenLoader = location.screens[j]
                    var screenName = Utils.qmlObjId(screenLoader)
                    if (screenName.toLowerCase().indexOf(ftext) !== -1) {
                        model.append({name: screenName, screenIndex: j,
                                      location: location, locationName: locationName})
                        if (screenLoader.screen) {
                            list.currentIndex = model.count - 1
                        }
                    }
                }
            }
        }
    }

    onVisibleChanged: {
        if (visible) {
            model.refill()
        } else {
            debugMenu.forceActiveFocus() // Return focus to debug
        }
    }

    ListView {
        id: list
        anchors.fill: parent; anchors.topMargin: 25
        anchors.bottomMargin: 58; anchors.leftMargin: 15; anchors.rightMargin: 15
        model: model
        clip: true
        focus: true
        highlight: Rectangle { width: 370; color: "LightSteelBlue" }

        section.property: "locationName"
        section.delegate: Text {
            width: 360; text: section; color: "DarkCyan"
        }

        delegate: Text {
            width: 340; height: 20; x: 40; font.pixelSize: 14; text: name
            Text { x: -30; font.pixelSize: 14; text: screenIndex}
        }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                var i = list.indexAt(mouseX, mouseY + list.contentY)
                if (i >= 0) {
                    list.currentIndex = i
                }
            }
            onDoubleClicked: {
                var i = list.indexAt(mouseX, mouseY + list.contentY)
                if (i >= 0) {
                    list.currentIndex = i
                    navigateButton.clicked()
                }
            }
        }

    }

    DebugButton {
        id: navigateButton
        anchors.bottom: parent.bottom; anchors.left: parent.left
        anchors.bottomMargin: 16; anchors.leftMargin: 35; width: 190
        text: "Navigate"
        onClicked: {
            debug.closeAll()
            var i = model.get(list.currentIndex)
            i.location.state = "Inactive"
            i.location.activeScreenIndex = i.screenIndex
            game.switchToLocation(i.location)
        }
    }

    DebugButton {
        anchors.bottom: parent.bottom; anchors.right: parent.right
        anchors.bottomMargin: 16; anchors.rightMargin: 35; width: 120
        text: "Close"
        selectedColor: "Brown"
        onClicked: debug.closeAll()
    }
}
