import QtQuick 1.1
import Common 1.0
import Qt.labs.canvas 1.0

// Highlights Obj-s for debug purpose
Canvas {

    property color strokeStyle: "Blue"
    property color fillStyle: "Cyan"

    opacity: 0.2
    anchors.fill: parent
    color: Qt.rgba(0, 0, 0, 0)

    onVisibleChanged: if (visible) requestPaint()

    Connections {
        target: game
        onScreenLoaded: if (visible) requestPaint()
        onZoomZoneStateChanged: if (visible) timerRequestPaint.start()
        onLocationChanged: if (visible) requestPaint() // Hide when menu opened
        onIspyFound: if (visible) timerRequestPaint.start()
        onIspyGroupEnabledChanged: if (visible) timerRequestPaint.start()
    }

    // Wait for animation
    Timer {
        id: timerRequestPaint
        interval: 700
        onTriggered: requestPaint()
    }

    onPaint: {
        var context = getContext("2d")
        context.clearRect(0, 0, width, height)
        context.beginPath()
        context.strokeStyle = strokeStyle
        context.fillStyle = fillStyle
        context.lineWidth = 2

        function draw(obj) {
            context.beginPath()
            if (obj.pickingShape.points.length === 0) {
                var p0 = game.mapFromItem(obj, 0, 0)
                var p1 = game.mapFromItem(obj, obj.width, 0)
                var p2 = game.mapFromItem(obj, obj.width, obj.height)
                var p3 = game.mapFromItem(obj, 0, obj.height)
                context.moveTo(p0.x, p0.y)
                context.lineTo(p1.x, p1.y)
                context.lineTo(p2.x, p2.y)
                context.lineTo(p3.x, p3.y)
                context.lineTo(p0.x, p0.y)
            } else {
                for (var k = 0; k < obj.pickingShape.points.length; ++k) {
                    var v = obj.pickingShape.polygonVertices(k)
                    var p = game.mapFromItem(obj, v[v.length - 2], v[v.length - 1])
                    context.moveTo(p.x, p.y)
                    for (var i = 0; i < v.length; i += 2) {
                        p = game.mapFromItem(obj, v[i], v[i + 1])
                        context.lineTo(p.x, p.y)
                    }
                }
            }
            context.fill()
            context.stroke()
        }

        function drawHierarchy(item) {
            if (item.visible && item.opacity && item.scale) {

                if (item.debugHighlightEnabled) {
                    draw(item)
                }

                for (var i = 0; i < item.children.length; ++i) {
                    drawHierarchy(item.children[i])
                }
            }
        }

        drawHierarchy(game)
    }
}
