import QtQuick 1.1

/// Adds short debug facilities to the application
Item {
    // Debug children must be DebugMenuButtons
    default property alias buttons: debugMenu.buttons
    
    property alias debugMenu: debugMenu
    
    property alias fps: fps
    
    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading
    
    id: self
    x: visibleArea.x; y: visibleArea.y
    width: visibleArea.width; height: visibleArea.height
    z: 15000
    visible: developmentConfig.get("debugEnabled") === "true"
    
    
    Item {
        id: debugInfo
        anchors.fill: parent
        visible: developmentConfig.get("showDebugInfo") === "true"
        
        Text {
            id: fps
            anchors.right: parent.right
            anchors.rightMargin: 10
            color: "Red"
            text: root.fps
            font.pixelSize: 14
        }
    }
    
    DebugMenu {
        id: debugMenu
        z: 100
        
        DebugMenuButton {
            text: "Skip all (F)"
            onClicked: skipPuzzle()
            hotKey: Qt.Key_F
        }
        
        DebugMenuButton {
            text: "Obj highlights (H)"; hotKey: Qt.Key_H
            target: ObjHighlightsAndInfo {}
        }
        
        DebugMenuButton {
            text: "Navigation (L)"; hotKey: Qt.Key_L
            target: Navigation {}
        }	
        
    }
    
    Component.onCompleted: {
        // Scan children for DebugMenuButton-s
        for (var i = 0; i < buttons.length; ++i) {
            var button = buttons[i]
            if (button.hotKey !== undefined && button.target) {
                button.target.visible = false
                button.target.parent = self
            }
        }
        
        // Focus self to work around the focus problem after reload
        self.forceActiveFocus()
        debugMenu.forceActiveFocus()
    }
    
    // Closes all debug windows except 'skip' one and menu, fps
    function closeAll(skip) {
        for (var i = 0; i < children.length; ++i) {
            var child = children[i]
            if (child !== skip && child !== debugMenu && child !== debugInfo) {
                child.visible = false
            }
        }
    }
    
}

