import QtQuick 1.1
import Common 1.0

Item {
    property real closeAngle

    // Cover image sources
    property url shadowSource: "Images/Shadow.png"
    property url coverLeftSource: "Images/CoverLeft.png"
    property url coverCenterSource: "Images/CoverCenter.png"
    property url coverRightSource: "Images/CoverRight.png"
    property url sheetsLeftSource: "Images/SheetsLeft.png"
    property url sheetsRightSource: "Images/SheetsRight.png"
    property url edgeLeftSource: "Images/EdgeLeft.png"
    property url edgeRightSource: "Images/EdgeRight.png"
    property url closeButtonSource: "Images/CloseButton.png"

    property alias closeButton: closeButton

    // Edge position when the cover is closed
    property real edgeShowAngle: 60
    property real edgeMarginX: 3
    property real edgeMarginY: 10
    property real edgeScaleMin: 1.5
    property real edgeScaleDelta: 0.1

    id: self
    width: imgLeft.width + imgCenter.width + imgRight.width
    height: Math.max(imgLeft.height, imgCenter.height, imgRight.height)

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: shadow
        source: shadowSource
        anchors.centerIn: parent
        opacity: closeAngle < 45 ? (45 - closeAngle) / 45 : 0
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: imgLeft
        source: coverLeftSource
        transform: Rotation {
            axis { x: 0; y: 1; z: 0 }
            origin { x: imgLeft.width; y: imgLeft.height * 0.5 }
            angle: self.closeAngle
        }
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: imgCenter
        anchors.left: imgLeft.right
        source: coverCenterSource
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: imgRight
        anchors.left: imgCenter.right
        source: coverRightSource
        transform: Rotation {
            axis { x: 0; y: 1; z: 0 }
            origin { x: 0; y: imgRight.height * 0.5 }
            angle: -self.closeAngle
        }
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: closeButton
        source: closeButtonSource
        anchors.right: imgRight.left
        anchors.top: parent.bottom
        anchors.rightMargin: -10
        anchors.topMargin: -35
        z: 1
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: sheetsLeft
        source: sheetsLeftSource
        anchors.right: parent.horizontalCenter
        z: 2
        transform: [
            Rotation {
                axis { x: 0; y: 1; z: 0 }
                origin { x: sheetsLeft.width; y: sheetsLeft.height * 0.5 }
                angle: self.closeAngle
            },
            Scale {
                origin { x: sheetsLeft.width; y: sheetsLeft.height * 0.5 }
                xScale: 1 - 0.1 * self.closeAngle / 90
            }
        ]
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: sheetsRight
        source: sheetsRightSource
        anchors.left: parent.horizontalCenter
        z: 2
        transform: [
            Rotation {
                axis { x: 0; y: 1; z: 0 }
                origin { x: 0; y: sheetsRight.height * 0.5 }
                angle: -self.closeAngle
            },
            Scale {
                origin { x: 0; y: sheetsRight.height * 0.5 }
                xScale: 1 - 0.1 * self.closeAngle / 90
            }
        ]
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: edgeLeft
        source: edgeLeftSource
        opacity: self.closeAngle > edgeShowAngle ? (self.closeAngle - edgeShowAngle) / (90 - edgeShowAngle) : 0
        scale: self.closeAngle > edgeShowAngle ? edgeScaleMin + edgeScaleDelta * (self.closeAngle - edgeShowAngle) / (90 - edgeShowAngle) : edgeScaleMin
        transformOrigin: Item.Left
        y: edgeMarginY
        x: edgeMarginX + imgLeft.width * (0.25 + 0.75 * (self.closeAngle - 60) / 30)
        z: 100
    }

    Image {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: edgeRight
        source: edgeRightSource
        opacity: edgeLeft.opacity
        scale: edgeLeft.scale
        transformOrigin: Item.Right
        y: edgeMarginY
        x: self.width - edgeRight.width - (edgeMarginX + imgRight.width * (0.25 + 0.75 * (self.closeAngle - 60) / 30))
        z: 100
    }
}
