import QtQuick 1.1
import Common 1.0

// Host for scalable areas when they are zoomed
ObjBase {
    property real _initialX
    property real _initialY
    property real _initialRotation
    property real _targetX
    property real _targetY
    property QtQuickItem _scalableArea

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: self
    anchors.fill: parent
    visible: false
    hoverEnabled: true

    function accept(scalableArea) {
        if (_scalableArea) {
            console.log("ScalableAreaZoomHost accepts only one scalable area")
            return
        }

        _scalableArea = scalableArea
        var p = self.mapFromItem(scalableArea, 0, 0)
        scalableArea.x = _initialX = p.x
        scalableArea.y = _initialY = p.y
        _initialRotation = scalableArea.rotation
        _targetX = (width - scalableArea.width) * 0.5
        _targetY = (height - scalableArea.height) * 0.5
        scalableArea.parent = self

        self.visible = true
        zoomInAnimation.start()
    }

    onClicked: {
        zoomInAnimation.stop()
        zoomOutAnimation.start()
    }

    ParallelAnimation {
        id: zoomInAnimation
        NumberAnimation {
            target: _scalableArea; properties: "x"; duration: 500; easing.type: Easing.InOutCubic; to: _targetX
        }
        NumberAnimation {
            target: _scalableArea; properties: "y"; duration: 500; easing.type: Easing.InOutCubic; to: _targetY
        }
        NumberAnimation {
            target: _scalableArea; properties: "rotation"; duration: 500; easing.type: Easing.InOutCubic; to: 0
        }
        NumberAnimation {
            target: _scalableArea; properties: "scale"; duration: 500; easing.type: Easing.InOutCubic
            to: _scalableArea ? _scalableArea.maxScale : 1
        }
    }


    ParallelAnimation {
        id: zoomOutAnimation
        NumberAnimation {
            target: _scalableArea; properties: "x"; duration: 500; easing.type: Easing.InOutCubic; to: _initialX
        }
        NumberAnimation {
            target: _scalableArea; properties: "y"; duration: 500; easing.type: Easing.InOutCubic; to: _initialY
        }
        NumberAnimation {
            target: _scalableArea; properties: "rotation"; duration: 500; easing.type: Easing.InOutCubic; to: _initialRotation
        }
        NumberAnimation {
            target: _scalableArea; properties: "scale"; duration: 500; easing.type: Easing.InOutCubic
            to: _scalableArea ? _scalableArea.minScale : 1
        }

        onCompleted: {
            self.visible = false
            var p = self.mapToItem(_scalableArea._initialParent, _initialX, _initialY)
            _scalableArea.x = p.x
            _scalableArea.y = p.y
            _scalableArea.parent = _scalableArea._initialParent
            _scalableArea = null
        }
    }
}
