import QtQuick 1.1

// A sheet consists of two pages.
// Turning effect supported thru turn property.
Item {
    property alias front: front
    property alias back: back
    property alias turnAnimation: turnAnimation
    // The properties below are read-only for the user code
    property real turn
    property int sheetIndex: -1
    property Item prevSheet
    property Item nextSheet

    id: sheet
    z: (sheetIndex + 1) * ((turn < 0) - (turn > 0))
    anchors.centerIn: parent
    width: front.width + back.width
    height: Math.max(front.height, back.height)

    onTurnChanged: {
        if (sheetIndex >= 0) {
            if (turn === 1) {
                freeSheetAt(nextSheetIndex(sheetIndex))
            } else if (turn === -1) {
                freeSheetAt(prevSheetIndex(sheetIndex))
            } else if (turn >= 0) {
                createSheetAt(nextSheetIndex(sheetIndex), 1)
            } else { // turn < 0
                createSheetAt(prevSheetIndex(sheetIndex), -1)
            }
        }
    }

    onSheetIndexChanged: if (sheetIndex < 0) {
        prevSheet = nextSheet = null
    }

    function animateToCorrectPosition() {
        turnAnimation.from = turn
        turnAnimation.to = currentSheetIndex > sheetIndex ? -1 : 1
        turnAnimation.duration = 400 * Math.abs(turnAnimation.to - turn)
        if (turnAnimation.duration > 0) {
            turnAnimation.restart()
        }
    }


    NumberAnimation {
        id: turnAnimation
        target: sheet; property: "turn"
    }

    Connections {
        target: sheetIndex >= 0 ? diary : null
        onCurrentSheetIndexChanged: animateToCorrectPosition()
    }

    Connections {
        target: sheetIndex >= 0 ? sheets : null
        onDataChanged: {
            prevSheet = sheets.at(sheetIndex - 1)
            nextSheet = sheets.at(sheetIndex + 1)
        }
    }

    // The front page is a right part of the sheet area
    Page {
        id: front
        visible: turn >= 0 && !(prevSheet && prevSheet.turn > turn - 0.001)
        anchors.left: back.right
        source: pageRightSource
        pageIndex: 2 * sheetIndex + 1
        upperSheetTurn: prevSheet ? 1 - prevSheet.turn : 2
    }
    // The back page is a left part of the sheet area
    Page {
        id: back
        visible: turn < 0 && !(nextSheet && nextSheet.turn < turn + 0.001)
        source: pageLeftSource
        pageIndex: 2 * sheetIndex + 2
        upperSheetTurn: nextSheet ? 1 + nextSheet.turn : 2
    }
}
