import QtQuick 1.1

// Default fading effect between screens.
Rectangle {
    property alias loadingIndicator: loadingIndicator

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    property bool _fadeoutStartedFired

    signal fadeoutStarted()


    id: self
    anchors.fill: parent
    color: "Black"
    opacity: 0

    PropertyAnimation {
        id: animation
        target: self; properties: "opacity"
        from: 1; to: 0
        duration: 500
        paused: !Utils.asyncLoading.isQueueEmpty // Cover the screen while loading

        onPausedChanged: _fireFadeoutStarted()
    }

    Connections {
        target: game
        onLocationChanged: if (!activeLocation) _startAnimation() // for menu
        onScreenLoadingStarted: self.opacity = 1
        onScreenLoaded: _startAnimation()
    }

    // Block input while effect running
    MouseArea {
        anchors.fill: parent
        visible: opacity > 0
    }

    Image {
        id: loadingIndicator
        source: "Images/Loading.png"
        //x: visibleArea.x + visibleArea.width - 70; y: visibleArea.y + visibleArea.height - 70
        anchors.centerIn: parent
        asynchronous: true

        PropertyAnimation on rotation {
            from: 0; to: 360
            duration: 2000; loops: PropertyAnimation.Infinite
            running: self.opacity != 0
        }
    }

    function _startAnimation() {
        _fadeoutStartedFired = false
        animation.restart()
        Utils.asyncLoading.checkQueue()

        _fireFadeoutStarted()
    }

    function _fireFadeoutStarted() {
        if (!animation.paused && !_fadeoutStartedFired) {
            fadeoutStarted()
            _fadeoutStartedFired = true
        }
    }
}
