.pragma library



function findJump(screenLoader) {
    var jumps = []
    _findJumpsOnScreen(screenLoader.screen, jumps)

    // Fill depths
    var jumpDepth = {}
    for (var i = 0; i < jumps.length; ++i) {
        var jump = jumps[i]
        jumpDepth[jump] = _findHintableScreenDepth([jump.toScreen], [screenLoader], [1])
    }

    // Find min depth
    var minDepthJumpIndex = -1, minDepth = 10000
    for (i = 0; i < jumps.length; ++i) {
        jump = jumps[i]
        if (jumpDepth[jump] < minDepth) {
            minDepthJumpIndex = i
            minDepth = jumpDepth[jump]
        }
    }

    return minDepthJumpIndex >= 0 ? {obj: jumps[minDepthJumpIndex]} : null
}


// Breadth-first traversing
function _findHintableScreenDepth(screenLoaders, visitedScreens, depths) {

    while (screenLoaders.length > 0) {
        var screenLoader = screenLoaders.shift()
        var depth = depths.shift()
        if (screenLoader.hintable) { // Assume we only change hintable to true in the game process.
            return depth
        }

        // Load screen to set hintable property
        var screen = screenLoader.loadScreenInContext()
        screenLoader.hintable = findHintOnScreen(screen) !== null
        if (screenLoader.hintable) {
            screenLoader.destroyScreen(screen)
            return depth
        }

        var jumps = []
        //console.debug(">>>>>>>>>>>>>>>>>>>>>>>>>>>")
        _findJumpsOnScreen(screen, jumps)
        //console.debug("<<<<<<<<<<<<<<<<<<<<<<<<<<Num jumps", screen, jumps.length)
        screenLoader.destroyScreen(screen)

        for (var i = 0; i < jumps.length; ++i) {
            var s = jumps[i].toScreen
            if (visitedScreens.indexOf(s) === -1) {
                screenLoaders.push(s)
                depths.push(depth + 1)
                visitedScreens.push(s)
            }
        }
    }
    return 10000
}


function _findJumpsOnScreen(item, jumps) {
    //console.debug(item.toScreen)
    if (item.toScreen && item.hintable && Utils.isVisibleTo(item, item.parent) && item.enabled) {
        jumps.push(item)
    }

    for (var i = 0; i < item.children.length; ++i) {
        var child = item.children[i]
        if (child.opacity > 0) {
            _findJumpsOnScreen(child, jumps)
        }
    }
}


function findHintOnScreen(screen) {
    var results = []
    _findHintOnScreenRecursive(screen, results)
    if (results.length === 0) {
        return null
    }
    return results[Math.floor(Math.random() * results.length)]
}


function findHintInBeltZoomZones(beltPackContainer) {
    var results = []
    for (var i = 0; i < beltPackContainer.children.length; ++i) {
        var child = beltPackContainer.children[i]
        if (child.zoomStates !== undefined) {
            _findHintOnScreenRecursive(child, results)
        }
    }
    if (results.length === 0) {
        return null
    }
    return results[Math.floor(Math.random() * results.length)]
}


function _findHintOnScreenRecursive(item, results) {
    if (item.enabled) {
        if (Utils.isVisibleTo(item, item.parent)) {
            var acceptedApplyObjs = item.acceptedApplyObjs
            if (acceptedApplyObjs !== undefined) {
                for (var k = 0; k < acceptedApplyObjs.length; ++k) {
                    var acceptedApplyObj = acceptedApplyObjs[k]
                    if (acceptedApplyObj._realObj) {
                        acceptedApplyObj = acceptedApplyObj._realObj // Support proxy
                    }
                    if (acceptedApplyObj.beltPackCollectable) {
                        if (beltPack.container.items.indexOf(acceptedApplyObj) !== -1) {
                            results.push({obj: item, applyObj: acceptedApplyObj, zoomZone: _findZoomZone(item)})
                            return
                        }
                    } else if (acceptedApplyObj.enabled
                               && acceptedApplyObj.opacity > 0
                               && Utils.isVisibleTo(acceptedApplyObj, acceptedApplyObj.parent)) {
                        results.push({obj: item, applyObj: acceptedApplyObj, zoomZone: _findZoomZone(item)})
                        return
                    }
                }
            }

            if (item.hintable
                    && item.toScreen === undefined // Do not hint jumps here
                    ) {
                results.push({obj: item, zoomZone: _findZoomZone(item)})
                return
            }

            if (item.puzzleSolved !== undefined && item.state === "PuzzleOn") {
                results.push({obj: item, zoomZone: _findZoomZone(item)})
                return
            }
        }

        for (var i = 0; i < item.children.length; ++i) {
            var child = item.children[i]
            if (child.opacity > 0) {
                _findHintOnScreenRecursive(child, results)
            }
        }
    }
}


function _findZoomZone(item) {
    for (var z = item.parent; z; z = z.parent) {
        if (z.zoomStates !== undefined)
            return z
    }
}

