import QtQuick 1.1

// Groups ispy objects.
Item {

    signal objectFound(variant ispy)

    property alias listModel: listModel
    property int numFound: 0

    id: self

    Connections {
        target: parent
        onScreenLoaded: if (enabled) makeEnable(true)
        onScreenLeft: makeEnable(false)
    }

    onEnabledChanged: {
        makeEnable(enabled)
        game.ispyGroupEnabledChanged(self)
    }

    Component.onCompleted: {
       fillList()
    }
    
    function fillList(){
        var index = 0
        for (var i = 0; i < resources.length; ++i) {
            var ispy = resources[i]
            if (ispy.group !== undefined) {
                ispy.group = self
                ispy.index = index++
                listModel.append({name: ispy.name, found: ispy.found, obj: ispy.obj,
                                     index: ispy.index, ispy: ispy})
                ispy.obj.debugHighlightEnabled = false
            }
        }   
    }

    ListModel {
        id: listModel
    }

    // Retets found state
    function reset() {
        for (var i = 0; i < listModel.count; ++i) {
            var ispy = listModel.get(i).ispy
            ispy.obj.state = ""
            ispy.found = false
        }
    }

    // Do not use this function directly!
    // Helps to enable/disable group.
    function makeEnable(enabled) {
        if (enabled) {
            beltPack.ispyList = listModel
        } else if (beltPack && beltPack.ispyList === listModel) {
            beltPack.ispyList = null
        }
        for (var i = 0; i < listModel.count; ++i) {
            listModel.get(i).obj.debugHighlightEnabled = enabled
        }
    }
}
