import QtQuick 1.1
import Common 1.0

// Button made of image collection.
// Can be used in a toggle button mode by adding the checked image.
ObjBase {
    property alias normalImage: normalImage
    property alias checkedImage: checkedImage
    property alias disabledImage: disabledImage
    property alias hoverImage: hoverImage
    property alias pressedImage: pressedImage
    property alias textComponent: textComponent
    property bool isChecked
    
    property real paddingLeft
    property real paddingRight
    property real paddingTop
    property real paddingBottom
    
    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: self
    //If image checked from start width and height have incorrect value in old version
    width: (normalImage.width || checkedImage.width) + (paddingLeft + paddingRight)
    height: (normalImage.height || checkedImage.height) + (paddingBottom + paddingTop)
    hoverEnabled: !game.hoverEnabled

    Item {
        anchors.fill: parent
        anchors.topMargin: paddingTop
        anchors.leftMargin: paddingLeft
        anchors.rightMargin: paddingRight
        anchors.bottomMargin: paddingBottom

        Image { id: normalImage;    visible: !containsMouse && !isChecked && self.enabled }
        Image { id: checkedImage;   visible: !containsMouse && isChecked && self.enabled }
        Image { id: disabledImage;  visible: !self.enabled }
        Image { id: hoverImage;     visible: containsMouse && !pressed }
        Image { id: pressedImage;   visible: pressed;   source: hoverImage.source }

        Text { id: textComponent; anchors.centerIn: parent;   opacity: self.enabled ? 1 : 0.4 }
    }

    onClicked: if (checkedImage.source != "") isChecked = !isChecked
}
