import QtQuick 1.1
import Common 1.0

// An object that is usually collected to the belt pack and often combined to the composition object.
// InventoryObj-s can also share a belt pack slot
ApplyObj {

    // Reference to InventoryObjComposition
    property QtObject composition

    // An object in the belt pack with the same composition. Filled automatically.
    property ApplyObj predecessor

    id: self
    beltPackCollectable: true
    visible: false

    // Overridden
    function putToBeltPack() {
        if (beltPackCollectable && state == "") {
            findPredecessor()
            state = predecessor ? "InBeltPackSharedSlot" : "InBeltPack"
            beltPack.itemAdded(self)
            // ApplyObj from another screen or location can be put to the belt pack, so, load its images
            Utils.setInactive(self, false)
        }
    }

    // Overridden
    function returnAfterDrag() {
        state = beltPackCollectable ? (predecessor ? "InBeltPackSharedSlot" : "InBeltPack") : ""
    }

    // Finds the object predecessor in the belt pack and sets predecessor property
    function findPredecessor() {
        var p = null
        if (composition && composition.sharedSlot) {
            var items = beltPack.container.children
            for (var i = 0; i < items.length && !p; ++i) {
                if (items[i].composition === composition) {
                    p = items[i]
                }
            }
        }
        predecessor = p
    }

    onStateChanged: if (composition) composition.pieceStateChanged(self)

    Component.onCompleted: {
        if (composition) {
            composition.pieces.append(self)
        }
    }
}
