import QtQuick 1.1
import Common 1.0

// Composition of InventoryObj in the belt pack
InventoryObj {

    // Fired when all pieces collected
    signal combined
    // Fired when a piece state changed
    signal pieceStateChanged(variant piece)

    // Indicates whether the pieces should be combined when all of them are collected
    property bool combineWhenAllPiecesCollected: true

    // If true the all pieces are placed in the same belt pack slot
    property bool sharedSlot

    // Composition counter. The user must not set this property.
    property int counter: 0
    // Filled automatically. The user must not set this property.
    property ObjectContainer pieces: ObjectContainer {}

    // Helps to build the counter text. Readonly.
    property string counterText: counter + "/" + pieces.count()
    property string counterText2: counter > 1 ? counter : ""


    onPieceStateChanged: timer.restart()
    // InventoryObj.state cannot be changed in onPieceStateChanged (qml limitaton), so, use the timer
    Timer {
        id: timer
        interval: 1
        onTriggered: {
            countPieces()
            if (combineWhenAllPiecesCollected && counter >= pieces.count()) {
                combine()
            }
        }
    }

    // Makes counter property valid
    function countPieces() {
        var c = 0
        for (var i = pieces.count(); --i >= 0;) {
            c += pieces.at(i).state.search(/InBeltPack\w*/) !== -1
        }
        counter = c
    }

    // Sets "Applied" for all pieces and puts the obj to the belt pack
    function combine() {
        for (var i = pieces.count(); --i >= 0;) {
            pieces.at(i).state = "Applied"
        }
        putToBeltPack()
        combined()
    }
}
