import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0

// Lamp control paremeters.
QtObject {
    
    id: lampObj
    
    property url highlightSource
    property url highlightAddSource
    property url raySource
    property real rayOriginX
    property real rayOriginY
    
    property real lampX
    property real lampY
    // Turns on/off
    property bool active
    property bool followDrag: true
    
    property real highlightWidth: 600
    property real highlightHeight: 600
    
    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading
    
    property url darkMapSource // Set by LampDarkMap
    
    onFollowDragChanged: {
        if (followDrag) {
            moveRay(screenDragX, screenDragY, 200)
        }
    }
    
    function moveRay(toX, toY, duration) {
        lampXAni.to = toX
        lampYAni.to = toY
        lampXAni.duration = duration
        lampYAni.duration = duration
        lampXAni.restart()
        lampYAni.restart()
    }
    
    NumberAnimation on lampX { id: lampXAni }
    NumberAnimation on lampY { id: lampYAni }
    
    property Binding _bindingLampX: Binding {
        target: lampObj; property: "lampX"
        value: screenDragX; when: followDrag && !lampXAni.running
    }
    property Binding _bindingLampY: Binding {
        target: lampObj; property: "lampY"
        value: screenDragY; when: followDrag && !lampYAni.running
    }

    // Declares dark effect at Screen. The item will be reparented on screen loaded.
    property QtQuickItem darkVisualItem: QtQuickItem {

        anchors.fill: parent
        z: 1000
        visible: parent !== null

        ObjBase { // ObjBase (not MouseArea) to overload cursors
            anchors.fill: parent
            visible: !active && darkMapSource != "" // Block input
        }

        ShaderEffectItem {

            property variant darkMap: QtQuickImage { source: darkMapSource; asynchronous: true }
            property variant highlight: QtQuickImage { source: highlightSource; asynchronous: true }
            property variant highlighTcMultiplier: Qt.point(width / highlightWidth, height / highlightHeight)
            property variant highlighTcSummand: Qt.point(-lampX / highlightWidth + 0.5 - (!active) * 100,
                                                         -lampY / highlightHeight + 0.5 - (!active) * 100)

            id: shaderEffect            
            width: game.width; height: game.height //anchors.fill: parent does not work here
            blending: true
            visible: darkMapSource != ""
            effect: CompositionMode { mode: CompositionMode.Multiply }

            fragmentShader: "
                varying mediump vec2 qt_TexCoord0;
                uniform sampler2D darkMap;
                uniform sampler2D highlight;
                uniform mediump vec2 highlighTcMultiplier;
                uniform mediump vec2 highlighTcSummand;
                void main() {
                    lowp vec4 dark = texture2D(darkMap, qt_TexCoord0);
                    lowp vec4 light = texture2D(highlight, qt_TexCoord0 * highlighTcMultiplier + highlighTcSummand);
                    gl_FragColor = dark + light;
                }"
        }
    }

    
    // Declares lamp ray effect at Screen. The item will be reparented on screen loaded.
    property QtQuickItem rayVisualItem: QtQuickItem {
        
        anchors.fill: parent
        z: 1000
        visible: parent !== null
                
        QtQuickImage {
            id: highLightAdd
            visible: active
            source: highlightAddSource
            width: highlightWidth; height: highlightHeight
            x: lampX - width * 0.5; y: lampY - height * 0.5
            effect: CompositionMode { mode: CompositionMode.Plus }
            asynchronous: true
        }
        
        QtQuickImage {
            id: rayImg
            property real dx: lampX - rayOriginX
            property real dy: lampY - rayOriginY
            visible: active
            source: raySource
            width: highlightWidth
            height: Math.sqrt(dx * dx + dy * dy)
            x: rayOriginX - width * 0.5
            y: rayOriginY - height
            transformOrigin: Item.Bottom
            rotation: Math.asin(dx / height) * 180 / Math.PI
            effect: CompositionMode { mode: CompositionMode.Plus }
            asynchronous: true
        }
    }
}
