import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0

// A surface with normal map lit by a point light source.
ShaderEffectItem {

    property Image diffuseMap: Image {}
    property Image normalMap: Image {} // Has specular mask in alpha channel
    property real specularPower: 7
    property real lightX
    property real lightY
    property real lightZ: 100
    property real lightRadius: 300
    property variant lightColor: Qt.rgba(1, 1, 1, 1)
    property variant ambientColor: Qt.rgba(0.3, 0.3, 0.3, 1)

    property variant _lightPos: Qt.vector3d(lightX / width, lightY / height, 2 * lightZ / (width + height))
    property real _lightRadius: 2 * lightRadius / (width + height)
    property variant _diffuseMapSampler: diffuseMap
    property variant _normalMapSampler: normalMap


    id: self
    width: normalMap.width
    height: normalMap.height
    blending: true
    shaderProgramName: "LitSurface" // Name to share program between ShaderEffectItem instances

    vertexShader: "
        uniform mediump mat4 qt_ModelViewProjectionMatrix;
        attribute mediump vec4 qt_Vertex;
        attribute mediump vec2 qt_MultiTexCoord0;
        varying mediump vec2 texCoord;
        void main(void)
        {
            texCoord = qt_MultiTexCoord0;
            gl_Position = qt_ModelViewProjectionMatrix * qt_Vertex;
        }"

    fragmentShader: "
        varying mediump vec2 texCoord;
        uniform lowp sampler2D _diffuseMapSampler;
        uniform lowp sampler2D _normalMapSampler;
        uniform lowp float qt_Opacity;
        uniform lowp vec4 lightColor;
        uniform lowp float specularPower;
        uniform lowp vec4 ambientColor;
        uniform lowp vec3 _lightPos;
        uniform lowp float _lightRadius;
        void main(void)
        {
            lowp vec4 dm = texture2D(_diffuseMapSampler, texCoord);
            lowp vec4 nm = texture2D(_normalMapSampler, texCoord);
            mediump vec3 point = vec3(texCoord.x, texCoord.y, 0);
            mediump float lightDist = distance(point, _lightPos);
            mediump float attenuation = 0.;
            lowp float lightCos = 0.;
            lowp vec4 specular = vec4(0.);
            if (lightDist < _lightRadius) {
                mediump vec3 normal = 2. * nm.rgb - vec3(1.);
                normal.y = -normal.y;
                normal = normalize(normal);
                attenuation = (_lightRadius - lightDist) / _lightRadius;
                attenuation = attenuation * attenuation;
                mediump vec3 lightDir = (point - _lightPos) / lightDist;
                mediump float specularStrength = max(reflect(lightDir, normal).z, 0.);
                specular.rgb = lightColor.rgb * pow(specularStrength, specularPower) * attenuation * nm.a * dm.a;
                lightCos = max(-dot(normal, lightDir), 0.);
            }
            gl_FragColor =  (dm * (ambientColor + lightColor * attenuation * lightCos) + specular) * qt_Opacity;
        }"
}
