import QtQuick 1.1
import Common 1.0

// Location component - base for locations.
QtQuickItem {
    property ScreenLoader cutScene
    property list<ScreenLoader> screens
    // Useful for save/load. Readonly.
    property int activeScreenIndex: 0
    // Active screen component. Readonly.
    property QtObject activeScreen: screens[activeScreenIndex].screen

    property alias screenContainer: screenZoom.container


    id: self
    anchors.fill: parent

    // Switches screen in the location
    function switchToScreen(screenLoader) {
        if (screens[activeScreenIndex].screen) {
            screens[activeScreenIndex].unload()
        } else if (cutScene && cutScene.screen) {
            cutScene.unload()
        }

        Utils.collectGarbage(root);

        if (screenLoader) {
            // Find activeScreenIndex
            for (var i = screens.length; --i >= 0;) {
                if (screens[i] === screenLoader) {
                    activeScreenIndex = i
                    break
                }
            }

            screenLoader.load()

            Utils._crtMemDifference()
            Utils._crtMemDumpStatistics("DIFFERENCE before switchToScreen")
            Utils._crtMemCheckpoint()
            Utils._crtMemDumpStatistics("before switchToScreen")
        }
    }

    onStateChanged: {
        if (state === "Inactive") {
            visible = false
            switchToScreen(null)
        } else {
            visible = true
            if (!cutScene || developmentConfig.get("skipCutscenes") === "true") {
                switchToScreen(screens[activeScreenIndex])
            } else {
                switchToScreen(cutScene)
            }
        }
    }

    // Allows to split saving/loading on files and also to zoom screen
    ScreenZoom {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading
        id: screenZoom
    }

    // Save/load support
    function saveState() {
        return activeScreenIndex > 0 ? {"activeScreenIndex" : activeScreenIndex} : {}
    }
    function loadState(data) {
        activeScreenIndex = data["activeScreenIndex"] || 0
    }
}
