import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0


ShaderEffectItem {

    property Image mask: Image {}
    property Image image: Image {}

    property variant maskScale: Qt.point(image.width / (mask.width * mask.scale), image.height / (mask.height * mask.scale))
    property variant maskOffset: Qt.point(mask.x / image.width, mask.y / image.height)
    property variant maskSampler: mask
    property variant imageSampler: image

    id: self
    width: image.width
    height: image.height
    blending: true
    shaderProgramName: "MaskedImage" // Name to share program between ShaderEffectItem instances

    vertexShader: "
        uniform mediump mat4 qt_ModelViewProjectionMatrix;
        attribute mediump vec4 qt_Vertex;
        attribute mediump vec2 qt_MultiTexCoord0;
        uniform mediump vec2 maskScale;
        uniform mediump vec2 maskOffset;
        varying mediump vec2 imageTexCoord;
        varying mediump vec2 maskTexCoord;
        void main(void)
        {
            maskTexCoord = (qt_MultiTexCoord0 - maskOffset) * maskScale;
            imageTexCoord = qt_MultiTexCoord0;
            gl_Position = qt_ModelViewProjectionMatrix * qt_Vertex;
        }"

    fragmentShader: "
        varying mediump vec2 imageTexCoord;
        varying mediump vec2 maskTexCoord;
        uniform lowp sampler2D maskSampler;
        uniform lowp sampler2D imageSampler;
        uniform lowp float qt_Opacity;
        void main(void)
        {
            lowp vec4 i = texture2D(imageSampler, imageTexCoord);
            lowp vec4 m = texture2D(maskSampler, maskTexCoord);
            gl_FragColor =  i * (m.r * qt_Opacity);
        }"
}
