import QtQuick 1.1
import Common 1.0


// Basic game object.
// Add Sprite or other visible element(s) as a child to appear the object.
ObjBase {

    // Fired when apply object accepted
    signal applyObjAccepted(variant applyObj)
    // Fired when apply object declined
    signal applyObjDeclined(variant applyObj)
    // List of accepted apply objects
    property list<QtObject> acceptedApplyObjs
    // List of hidden states that must be faded out
    property variant fadeoutStates: ["Done", "Found"]
    // Duration of hidden states fade-out
    property int fadeoutDuration: 300
    // If false the object is not highlighted in debug mode
    property bool debugHighlightEnabled: true
    // If true the object can be hinted. The object is also hinted if it has acceptedApplyObjs in the belt pack.
    property bool hintable: false
    // If the object was hinted and spendHint == false the hint value is not cleared.
    property bool spendHint: true

    property QtObject clickSound // ActionSoundGroup or ActionSound
    property QtObject applySound // ActionSoundGroup or ActionSound

    id: self
    // Bind size to chindren rect size
    width: childrenRect.width + childrenRect.x
    height: childrenRect.height + childrenRect.y

    // Removes apply obj from the list
    function removeAcceptedApplyObj(obj) {
        var newList = []
        var aao = acceptedApplyObjs
        for (var i = 0; i < aao.length; ++i) {
            if (aao[i] !== obj) {
                newList.push(aao[i])
            }
        }
        acceptedApplyObjs = newList
    }

    // Communication with apply objects
    function acceptApplyObj(applyObj) {
        var aao = acceptedApplyObjs
        for (var i = 0; i < aao.length; ++i) {
            // _realObj is a support for proxies
            if (aao[i] === applyObj || aao[i]._realObj === applyObj) {
                if (applySound) {
                    applySound.play()
                }
                applyObjAccepted(applyObj)
                game.applyObjAccepted(self, applyObj)
                return true
            }
        }
        applyObjDeclined(applyObj)
        game.applyObjDeclined(self, applyObj)
    }

    onClicked: {
       clickImpl(mouse)
    }
    
    function clickImpl(mouseItem){ //needed for Siren debug purpose (cheat apply obj on RightClickMouse)
        if (clickSound) {
            clickSound.play()
        }
        game.objClicked(self)
    }

    PropertyAnimation on opacity {
        duration: fadeoutDuration; to: 0
        running: fadeoutStates.indexOf(state) !== -1 && !Persistence.isLoading
    }

    onStateChanged: {
        // Restore opacity if change from fadeoutStates
        if (opacity == 0) {
            if (fadeoutStates.indexOf(state) === -1 && fadeoutStates.length > 0) {
                opacity = 1
                enabled = true
            }
        } else {
            // Disable if Done or Found
            if (fadeoutStates.indexOf(state) !== -1) {
                enabled = false
                if (Persistence.isLoading) {
                    opacity = 0
                }
            }
        }
    }
}
