import QtQuick 1.1
import Common 1.0

// Controls game persistence. Must be aggregated by game.
QtObject {
    // Fired when game loaded
    signal gameLoaded


    // Must be set by the user.
    property string profileName: "__Default__"

    // Disables save profile onScreenLeft and onScreenUnloaded
    property bool autoSaveDisabled


    property ProfileManager profileManager: ProfileManager {
        globalDataDir: Persistence.writableLocation
        profilesDir: Persistence.writableLocation + "/Profiles"

        onProfilesDirChanged: {
            Persistence.removeDir(profilesDir + "/__Default__")
        }
    }


    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading



    // Loads game state.
    function loadGame() {
        if (loadObject("Game", game)) {
            gameLoaded()
            return true
        }
        return false
    }

    // Force saving. May be useful to save application on significant events.
    function forceSave() {
        saveObject("Game", game)
        if (activeLocation && activeLocation.activeScreen) {
            saveObject(activeLocation.activeScreen.objectName, activeLocation.activeScreen)
        }
    }

    // Convenient way to clean a profile name
    function cleanName(name) {
        var normalizedName = name.replace(/\W/, ' ') // Replace non-word characters to space
        return normalizedName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') // Trim
    }

    function saveObject(name, object) {
        return profileManager.save(profileName, name, object)
    }

    function loadObject(name, object) {
        return profileManager.load(profileName, name, object)
    }


    // Returns true if profile with such name exists
    function profileExists(name) {
        return profileManager.profileNames.indexOf(name) !== -1
    }


    property Connections _persistenceControllerConnections: Connections {
        target: autoSaveDisabled ? null : game
        onScreenLeft:   saveObject(screen.objectName, screen)
        onScreenUnloaded: { saveObject("Game", game); profileManager.synchWithServer() }
    }
}
