import QtQuick 1.1
import Common 1.0

// Convenient component for pop-up windows.
// Implements zoom animation.
Obj {

    property real minOpacity
    property alias scaleAnimation: scaleAnimation
    property alias opacityAnimation: opacityAnimation

    // Set automatically
    property QtQuickItem initialParent

    property bool doNotCompleteAnimationsOnSave: true

    property string unserializableProperties: "state, scale, unserializableProperties"


    id: self
    state: "Closed"
    visible: scale > 0
    scale: 0
    opacity: minOpacity
    itemEnabled: opacity === 1 || scale === 0 // Make container enabled for hinting when scale === 0
    debugHighlightEnabled: false
    fadeoutStates: []

    Component.onCompleted: initialParent = parent

    onScaleChanged: {
        // Avoid binding to have initialParent initialized
        parent =  scale > 0 ? popupLayer : initialParent
    }

    function open() {
        state = "Opened"
    }

    function close() {
        state = "Closed"
    }

    // Work around of double changing of visible and scale on state change
    NumberAnimation {
        id: scaleAnimation
        target: self; property: "scale"
        to: state !== "Closed" ? 1 : 0
        onToChanged: if (to !== scale) restart()
        easing.type: Easing.InOutQuad; duration: 400
    }
    NumberAnimation {
        id: opacityAnimation
        target: self; property: "opacity"
        to: state !== "Closed" ? (parent && parent.topWindow === self) : minOpacity
        onToChanged: if (to !== opacity) restart()
        easing.type: Easing.InOutQuad; duration: 400
    }
}
