import QtQuick 1.1
import Common 1.0

// Base component for screens. The component works in conjunction with ScreenLoader.
QtQuickItem {

    // Fired when the game screen is loaded and visible.
    signal screenLoaded()
    // Fired when the game screen is left and hidden.
    signal screenLeft()

    // Fired when fading is comleted and images are loaded
    signal fadingCompleted()

    // Fired when fade out is started and images are loaded
    signal fadeoutStarted()

    // Indicates whether gui and beltpack are hidden when the screen active.
    property bool hiddenGuiAndBeltPack

    // Indicates whether screen zoom enabled
    property bool isScreenZoomEnabled: true

    property url lampDarkMapSource
    property QtQuickItem lampDarkMapHost: screen

    // Optimization for binding
    property bool isPopupWindowOpen: popupLayer.isWindowOpen

    // Indicates whether onScreenLoaded was fired
    property bool isLoaded

    property bool neighbourScreensPreloadingEnabled: true


    id: screen
    anchors.fill: parent

    onHiddenGuiAndBeltPackChanged: {
        gui.state = beltPack.state = hiddenGuiAndBeltPack ? "Inactive" : ""
    }

    onScreenLoaded: {
        if (lamp) {
            lamp.darkMapSource = lampDarkMapSource
            lamp.darkVisualItem.parent = lampDarkMapHost
            lamp.rayVisualItem.parent = screen
        }
        gui.state = beltPack.state = hiddenGuiAndBeltPack ? "Inactive" : ""
        isLoaded = true
    }
    onScreenLeft: {
        popupLayer.closeAllWindows()
        isLoaded = false
        gui.state = beltPack.state = "Inactive"
        if (lamp) {
            lamp.rayVisualItem.parent = null
            lamp.darkVisualItem.parent = null
            lamp.darkMapSource = ""
        }
    }

    onLampDarkMapSourceChanged: {
        if (lamp && lamp.darkVisualItem.parent === lampDarkMapHost) {
            lamp.darkMapSource = lampDarkMapSource
        }
    }

    // Scale screen with popup layer
    transform: Scale {
        xScale: popupLayer.zoomScale; yScale: popupLayer.zoomScale
        origin.x: popupLayer.zoomCenterX; origin.y: popupLayer.zoomCenterY
    }

    Connections {
        target: visible ? fading : null
        onOpacityChanged: if (fading.opacity === 0) {
            fadingCompleted()
            game.fadingCompleted(screen)
        }
        onFadeoutStarted: {
            fadeoutStarted()
            game.fadeoutStarted(screen)
        }
    }
}
