import QtQuick 1.1
import Common 1.0

// Zooms screen by double tap and pinch
FlickableExt {
    property alias container: container
    property real maxZoom: platformTraits & RootItem.TouchEnabledPlatform ? 1.5 : 1

    id: self
    anchors.fill: parent
    contentWidth: width * transformScale.xScale
    contentHeight: height * transformScale.yScale
    boundsBehavior: FlickableExt.StopAtBounds

    onMouseDoubleClick: {
        scaleTo(scaleAnimation.to === 1
                && container.children.length > 0 && container.children[0].isScreenZoomEnabled
                    ? maxZoom : 1, mouseX, mouseY)
    }

    function scaleTo(scale, originX, originY, norestart) {
        if (!norestart) {
            scaleAnimation.stop()
            contentXAnimation.stop()
            contentYAnimation.stop()
        }

        scaleAnimation.to = scale
        scaleAnimation.start()

        contentXAnimation.to = Math.min(Math.max(0, scale * (contentX + originX) / transformScale.xScale - originX),
                                        width * (scale - 1))
        contentXAnimation.start()

        contentYAnimation.to = Math.min(Math.max(0, scale * (contentY + originY) / transformScale.yScale - originY),
                                        height * (scale - 1))
        contentYAnimation.start()
    }

    function resetZoom(withoutAnimation) {
        scaleTo(1, width / 2, height / 2)
        if (withoutAnimation) {
            scaleAnimation.complete()
            contentXAnimation.complete()
            contentYAnimation.complete()
        }
    }

    Connections {
        target: game
        onObjClicked: resetZoom()
        onScreenLeft: resetZoom(true)
        onHintShown: resetZoom()
        onDiaryOpened: resetZoom()
        onApplyObjStateChanged: if (applyObj.state === "ReadyToApply") resetZoom()
    }

    Connections {
        target: popupLayer
        onTopWindowChanged: resetZoom(true)
    }

    Connections {
        target: root
        onPinch: {
            if (pinchScale > 1.2) {
                if (!popupLayer.isWindowOpen && !draggingApplyObj
                        && container.children.length > 0 && container.children[0].isScreenZoomEnabled)
                    scaleTo(maxZoom, pinchX, pinchY, true)
            } else if (pinchScale < 0.8) {
                scaleTo(1, pinchX, pinchY, true)
            }
        }
    }

    QtQuickItem {
        id: container
        anchors.fill: parent
        transform: Scale { id: transformScale }
    }

    NumberAnimation {
        id: scaleAnimation
        duration: 500; easing.type: Easing.InOutQuad; to: 1
        target: transformScale; properties: "xScale, yScale"
    }

    NumberAnimation {
        id: contentXAnimation
        duration: 500; easing.type: Easing.InOutQuad
        target: self; property: "contentX"
    }

    NumberAnimation {
        id: contentYAnimation
        duration: 500; easing.type: Easing.InOutQuad
        target: self; property: "contentY"
    }
}
