import QtQuick 1.1
import Audio 1.0

// Video with audio tracks and subtitles
OgvVideo {

    property list<Subtitle> subtitles
    property list<AudioTrack> audioTracks
    // Set automatically
    property Subtitle currentSubtitle
    property int currentSubtitleIndex

    id: self

    // Default subtitle template
    property Component subtitleTemplate : Text {
        text: currentSubtitle ? currentSubtitle.text : ""
        color: "White"
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.bottom: parent.bottom; anchors.bottomMargin: 20
    }

    Component.onCompleted: {
        subtitleTemplate.createObject(self)
    }

    onSubtitlesChanged: {
        currentSubtitle = null
        currentSubtitleIndex = 0
        _prepareSubtitles()
    }

    onStreamStarted: {
        if (playbackRate === 1) {
            stopAudioTracks()
            playAudioTracks(position)
        }
        findActiveSubtitle(position)
    }

    onPlaybackRateChanged: {
        if (playbackRate === 1) {
            playAudioTracks(position)
        } else {
            stopAudioTracks()
        }
    }

//    property int referenceTime
//    NumberAnimation on referenceTime {
//        id: referenceTimeAnimation
//        from: 0; to: 100000; duration: to
//        paused: !isApplicationFocused
//    }

    onPositionChanged: {
        findActiveSubtitle(position)

        // Synchronize video with audio
        if (audioTracks.length > 0) {
            var apos = audioTracks[0].playingOffset()
            if (apos > 0 && Math.abs(apos - position) > 100) {
                var c = (apos - position) * 0.0005
                c = Math.max(-0.4, Math.min(c, 0.4))
                playbackCorrection = 1 - c
            } else {
                playbackCorrection = 1
            }

            //console.debug("rt:", referenceTime, "a:", apos, "v:", position,
            //              apos  - referenceTime, position - referenceTime, playbackCorrection)
        }
    }

    function _prepareSubtitles() {
        // Set time in subtitles
        for (var i = 0, e = subtitles.length, time = 0; i < e; ++i) {
            var subtitle = subtitles[i]
            if (subtitle.time <= 0) {
                subtitle.time = time
            } else {
                time = subtitle.time
            }
            time += subtitle.duration
        }
    }

    function findActiveSubtitle(position) {

        for (var i = currentSubtitleIndex; i < subtitles.length; ++i) {
            var subtitle = subtitles[i]
            if (subtitle.time <= position && position < subtitle.time + subtitle.duration) {
                currentSubtitle = subtitles[currentSubtitleIndex = i]
                return
            }
        }

        for (i = 0; i < currentSubtitleIndex; ++i) {
            subtitle = subtitles[i]
            if (subtitle.time <= position && position < subtitle.time + subtitle.duration) {
                currentSubtitle = subtitles[currentSubtitleIndex = i]
                return
            }
        }

        currentSubtitle = null
    }

    function playAudioTracks(position) {
        //referenceTimeAnimation.restart()
        for (var i = 0; i < audioTracks.length; ++i) {
            audioTracks[i].play(position)
        }
    }

    function stopAudioTracks(position) {
        //referenceTimeAnimation.stop()
        for (var i = 0; i < audioTracks.length; ++i) {
            audioTracks[i].stop()
        }
    }
}
