import QtQuick 1.1

// Convenient component for ZoomZone containers.
PopupWindow {

    default property alias content: contentContainer.data
    property alias contentContainer: contentContainer
    property alias adorners: adornersContainer.data
    property alias adornersContainer: adornersContainer

    property real screenZoomScale: 1.4

    // Position for the closed container.
    property real closedX: zoomZone.parent.mapToItem(game, zoomZone.x, zoomZone.y).x + (zoomZone.width - width) * 0.5
    property real closedY: zoomZone.parent.mapToItem(game, zoomZone.x, zoomZone.y).y + (zoomZone.height - height) * 0.5

    // Position for the opened container.
    property real openedX: (parent.width - width) * 0.5
    property real openedY: (parent.height - height) * 0.5

    // Reference to the parent zoomZone. Set automatically.
    property Item zoomZone: initialParent

    id: self
    x: state === "Closed" ? closedX : openedX
    y: state === "Closed" ? closedY : openedY
    doNotCompleteAnimationsOnSave: false

    onStateChanged: {
        if (state === "Closed") {
            if (visible) {
                completeAnimationsWithDelay.start()
                popupLayer.zoomScale = 1
            }
        } else {
            // Scale screen and popup blur texture for screen zoom zones
            if (zoomZone.beltPackIndex === undefined) {
                var zcx = x + width * 0.5
                var zcy = y + height * 0.5
                // Slightly move the scale center to the edge of screen
                zcx = Math.max(0, Math.min(2 * (zcx - popupLayer.width * 0.5) + popupLayer.width * 0.5, popupLayer.width))
                zcy = Math.max(0, Math.min(2 * (zcy - popupLayer.height * 0.5) + popupLayer.height * 0.5, popupLayer.height))
                popupLayer.zoomCenterX = zcx
                popupLayer.zoomCenterY = zcy
                popupLayer.zoomScale = screenZoomScale
            }
        }
    }

    function open() {
        state = "Zoomed"
    }

    // Some animations change the zoom zone state, so delay it relative to onStateChanged
    PauseAnimation {
        id: completeAnimationsWithDelay
        duration: 50
        onCompleted: Utils.completeAnimationsRecursively(contentContainer)
    }

    Behavior on x {
        NumberAnimation { easing.type: Easing.InOutQuad; duration: 400 }
    }

    Behavior on y {
        NumberAnimation { easing.type: Easing.InOutQuad; duration: 400 }
    }

    Item {
        id: contentContainer
        anchors.fill: parent
        clip: true
    }

    Item {
        id: adornersContainer
        anchors.fill: parent
    }

}
