import QtQuick 1.1
import Common 1.0

Obj {
    id:self
    //Public===============================================================================================================================================
    property alias imageSource:selfimage.source
    property alias aliasAnimationAngle:selfRotation.angle

    property variant stateList: []                         // Набор состояний вращающегося элемента головоломки
    property int statesNumber:1                            // Количество состояний в stateList
    // ^^^Внимание! Если указано 0 то объект связывает при инициализации сигнал clicked() c аналогичным сигналом родителя!

    property int idNumber:0                                // Идентификационный номер объекта начинается с 0(Задается при создании).
    property int currentState:0                            // Текущий номер варианта в stateList
    property int finishState:0                             // Номер верного варианта stateList
    property int oldState:0                                // Предидущий номер варианта в stateList
    property real originX:0                                // Локальная координата X для точки вращения на изображении элемента головоломки
    property real originY:0                                // Локальная координата Y для точки вращения на изображении элемента головоломки
    property real originXdelta:self.parent.originXdelta+self.x
    property real originYdelta:self.parent.originYdelta+self.y
    property real animationAngle:stateList[currentState]   // Переменная - текущий угол поворота элемента головолмки
    //Private==============================================================================================================================================

    signal localStateNew()

    function statesArrayNew() {                            // Функция для внесения изменений в variant currentStates после изменения состояний объектов
        var temp=currentStates
        temp[idNumber]=currentState
        currentStates=temp
    }

    Image {id:selfimage;}

    transform: Rotation { id: selfRotation; origin.x:originX-originXdelta; origin.y:originY-originYdelta;  angle:animationAngle }

    onClicked: {
        if (!blockCkicks) {
            if (self.enabled) {
                if (statesNumber!=0) {
                    if (currentState!=statesNumber-1) {
                        oldState=currentState
                        currentState++
                        statesArrayNew()
                    }
                    else {oldState=currentState; currentState=0;statesArrayNew()}
                }
            }
        }
    }

    onLocalStateNew: {
        if (currentStates[self.idNumber]!==currentState){
            oldState=currentState
            currentState=currentStates[self.idNumber]
        }
    }

    onCurrentStateChanged: {
        animationAngle=stateList[currentState]
        if (currentState===finishState) currentFinishStatesNumber++
        else {if (oldState==finishState) currentFinishStatesNumber-- }
    }

    Behavior on animationAngle {
        SequentialAnimation {
            id:angleAnimation
            ScriptAction{script:{if (blockAnimatedElement) self.enabled=false}}
            NumberAnimation { duration: animationDuration }
            ScriptAction{script:{if (blockAnimatedElement) self.enabled=true}}
            onRunningChanged: {
                if (!angleAnimation.running) oneAnimationEnd()
                else {oneAnimationBegin(); if (!animationRun) {if (!notLimitedClicks) blockCkicks=true;animationRun=true} }
            }
        }
    }

    Component.onCompleted: {
        // Если для объекта параметр statesNumber установлен в 0 то данный объект не имеет состояний и перенаправляет клики своему объекту родителю
        if (statesNumber===0) {clicked.connect(parent.clicked);applyObjAccepted.connect(parent.applyObjAccepted)}
        else currentStatesNew.connect(localStateNew) //Соединяем сигналы localStateNew всех активных элементов головоломки с сигналом головоломки currentStatesNew
    }
}
