import QtQuick 1.1
import Common 1.0


Obj {

    // у всех SlideElement objectName должен содержать "slider"
    // у путей objectName должен содержать "forward"/"backward" плюс номер позиции

    id: slidePuzzle

    property variant sequence//: [1,4,2,3]
    property variant winPosition//: [0,3,2,1]
    property variant startPosition//: [1,0,2,3]
    property int winXOffset: 0
    property int winYOffset: -100
    property int emptyPos
    property int durationPerCurve: 100
    property bool solved: false

    onStartPositionChanged: {
        var i, j, k

        for (i = 0; i < startPosition.length; i++) {
            if (startPosition[i] === 0) {
                emptyPos = i+1
                continue
            }
            for (j = 0; j < resources.length; j++) {
                if (resources[j].objectName === "forward" + (i+1)) {
                    for (k = 0; k < children.length; k++) {
                        if (children[k].objectName === "slider" + startPosition[i]) {
                            children[k].startPos = startPosition[i]
                            children[k].currentPos = i+1
                            children[k].x = resources[j].startX
                            children[k].y = resources[j].startY
                            break;
                        }
                    }
                    break
                }
            }
        }
    }

    onWinPositionChanged: {
        var i, j, k

        for (i = 0; i < winPosition.length; i++) {
            if (winPosition[i] === 0) {
                continue
            }
            for (j = 0; j < resources.length; j++) {
                if (resources[j].objectName === "forward" + (i+1)) {
                    for (k = 0; k < children.length; k++) {
                        if (children[k].objectName === "winpos" + winPosition[i]) {
                            children[k].x = resources[j].startX + winXOffset
                            children[k].y = resources[j].startY + winYOffset
                            break;
                        }
                    }
                    break
                }
            }
        }
    }

    function solveCheck() {

        var check = true
        var i

        for (i = 0; i < children.length; i++) {
            if (children[i].enabled === false) {
                return
            }
        }

        for (i = 0; i < children.length; i++) {
            if (children[i].objectName.indexOf("slider") !== -1) {
                if (children[i].startPos !== winPosition[children[i].currentPos-1]) {
                    check = false
                    break;
                }
            }
        }
        if (check) {
            solved = true
        }
    }

    function slideStart(element) {

        var nextPos, prevPos, currentIndex, i, animName

        currentIndex = sequence.indexOf(element.currentPos)
        nextPos = sequence[(currentIndex === sequence.length-1 ? 0 : currentIndex+1)]
        prevPos = sequence[(currentIndex === 0 ? sequence.length-1 : currentIndex-1)]

        animName = ""
        if (nextPos === emptyPos) {
            animName = "forward" + element.currentPos
        } else if (prevPos === emptyPos) {
            animName = "backward" + element.currentPos
            nextPos = prevPos
        }

        if (animName !== "") {
            for (i = 0; i < resources.length; i++) {
                if (resources[i].objectName === animName) {
                    emptyPos = element.currentPos
                    element.currentPos = nextPos
                    element.pathAnim.path = resources[i]
                    element.enabled = false
                    element.slideDuration = resources[i].pathElements.length * durationPerCurve
                    element.pathAnim.restart();
                    break;
                }
            }
        }
    }

    /*    Image {
        objectName: "winpos1"
        source: "Slider1.png"
        scale: 0.30
    }

    Image {
        objectName: "winpos2"
        source: "Slider2.png"
        scale: 0.30
    }

    Image {
        objectName: "winpos3"
        source: "Slider3.png"
        scale: 0.30
    }

    Image {
        source: "Path.png"
        x:50
        y:50
    }

    SlideElement {
        objectName: "slider1"
        Image {source: "Slider1.png"}
    }

    SlideElement {
        objectName: "slider2"
        Image{source: "Slider2.png"}
    }

    SlideElement {
        objectName: "slider3"
        Image{source: "Slider3.png"}
    }

    Path {
        id: forward1
        objectName: "forward1"
        startX: 127; startY: 197
        PathCurve { x: 284; y: 106 }
        PathCurve { x: 468; y: 202 }
    }
    Path {
        id: backward1
        objectName: "backward1"
        startX: 123; startY: 202
        PathCurve { x: 143; y: 223 }
        PathCurve { x: 414; y: 338 }
        PathCurve { x: 551; y: 117 }
        PathCurve { x: 348; y: 201 }
    }
    Path {
        id: forward2
        objectName: "forward2"
        startX: 237; startY: 202
        PathCurve { x: 32; y: 183 }
        PathCurve { x: 205; y: 319 }
        PathCurve { x: 342; y: 204 }
    }
    Path {
        id: backward2
        objectName: "backward2"
        startX: 242; startY: 208
        PathCurve { x: 380; y: 272 }
        PathCurve { x: 470; y: 200 }
    }
    Path {
        id: forward3
        objectName: "forward3"
        startX: 348; startY: 201
        PathCurve { x: 556; y: 145 }
        PathCurve { x: 414; y: 338 }
        PathCurve { x: 123; y: 202 }
    }
    Path {
        id: backward3
        objectName: "backward3"
        startX: 342; startY: 204
        PathCurve { x: 109; y: 327 }
        PathCurve { x: 51; y: 120 }
        PathCurve { x: 237; y: 202 }
    }
    Path {
        id: forward4
        objectName: "forward4"
        startX: 470; startY: 200
        PathCurve { x: 354; y: 269 }
        PathCurve { x: 242; y: 208 }
    }
    Path {
        id: backward4
        objectName: "backward4"
        startX: 468; startY: 202
        PathCurve { x: 284; y: 106 }
        PathCurve { x: 127; y: 197 }
    }
    */
}
