import QtQuick 1.1
import Common 1.0

Obj {

    id: dragPuzzle

    property int minY: 0
    property int maxY: 400
    property real precision: 0.05

    property string sliderImg: ""
    property string beaconImg: ""

    property QtObject switchSound
    property bool winner

    signal sliderSolved(variant slider)

    property alias basicElement1: basicElement1
    property alias basicElement2: basicElement2
    property alias basicElement3: basicElement3

    property alias moveSound1: basicElement1.moveSound
    property alias moveSound2: basicElement2.moveSound
    property alias moveSound3: basicElement3.moveSound


    onSliderSolved: {
        if (basicElement1.sliderElement.solved && basicElement2.sliderElement.solved && basicElement3.sliderElement.solved && !winner){
            winner = true
        }
    }

    PuzzleElement {
        id: basicElement1
        sliderElement.x: 100
        sliderElement.y: 150
        //        solvingY: 50
        solvingPrecision: precision
        sliderImage.source: sliderImg
        sliderBeacon.cx: 650
        sliderBeacon.cy: 50
        beaconImage.source: beaconImg
        sliderArea.drag.minimumY: minY
        sliderArea.drag.maximumY: maxY
        //==The connection between elements. Plus or minus before ( is the vector of movement
        sliderElement.onYChanged: {
            if ((sliderElement.y == basicElement2.sliderElement.y)
                    && ((sliderElement.y == minY) || (sliderElement.y == maxY))){
                basicElement2.signIndex = -1
            }
            if ((sliderElement.y == minY && basicElement2.sliderElement.y == maxY)
                    ||(sliderElement.y ==  maxY && basicElement2.sliderElement.y == minY)) {
                basicElement2.signIndex = 1
            }
            if ((sliderElement.y == basicElement3.sliderElement.y)
                    && ((sliderElement.y == minY) || (sliderElement.y == maxY))){
                basicElement3.signIndex = -1
            }
            if ((sliderElement.y == minY && basicElement3.sliderElement.y == maxY)
                    ||(sliderElement.y ==  maxY && basicElement3.sliderElement.y == minY)) {
                basicElement3.signIndex = 1
            }
            if (sliderArea.drag.active) {
                if (basicElement3.visible) {
                    basicElement3.sliderElement.y = basicElement3.sliderElement.lastY - (sliderElement.lastY - sliderElement.y)
                }
            }
            sliderElement.lastY = sliderElement.y
        }
    }

    PuzzleElement {
        property int signIndex: 1
        id: basicElement2
        sliderElement.x: 250
        sliderElement.y: 150
        solvingPrecision: precision
        sliderImage.source: sliderImg
        sliderBeacon.cx: 650
        sliderBeacon.cy: 250
        beaconImage.source: beaconImg
        sliderArea.drag.minimumY: minY
        sliderArea.drag.maximumY: maxY
        sliderElement.onYChanged: {
            if (sliderArea.drag.active) {
                if (basicElement1.visible) {
                    basicElement1.sliderElement.y = basicElement1.sliderElement.lastY + (sliderElement.lastY - sliderElement.y)*signIndex
                }
            }
            sliderElement.lastY = sliderElement.y
        }
        onSignIndexChanged: {
            if (switchSound && dragPuzzle.visible) switchSound.play()
        }
    }

    PuzzleElement {
        property int signIndex: -1
        id: basicElement3
        sliderElement.x: 400
        sliderElement.y: 150
        solvingPrecision: precision
        sliderImage.source: sliderImg
        sliderBeacon.cx: 650
        sliderBeacon.cy: 450
        beaconImage.source: beaconImg
        sliderArea.drag.minimumY: minY
        sliderArea.drag.maximumY: maxY
        sliderElement.onYChanged: {
            if (sliderArea.drag.active) {
                if (basicElement1.visible) {
                    basicElement1.sliderElement.y = basicElement1.sliderElement.lastY + (sliderElement.lastY - sliderElement.y)*signIndex
                }
            }
            sliderElement.lastY = sliderElement.y
        }
        onSignIndexChanged: {
            if (switchSound && dragPuzzle.visible) switchSound.play()
        }
    }
    //======Puzzle start values
    //==Slider image (should exist)
    //        sliderImg: "../../../Werewolf/Location1/Village_Closet/ZZChest/Caduceus.png"
    //==Beacon image (could be empty)
    //        beaconImg: "../../../Werewolf/Location1/Village_Closet/ZZChest/RedSignal.png"
    //==Drag area (default - 0-400)
    //        minY: 0
    //        maxY: 400
    //==Slider position precision for solving (0.05 by default)
    //        precision: 0.05
    //==If you need some actions to activate the puzzle (all elements are visible by default)
    //        basicElement1.visible: false
    //==If you need to set the coordinates for the first element (100x150 are default)
    //        basicElement1.sliderElement.x: 100
    //        basicElement1.sliderElement.y: 150
    //==Solving position for the first element (by default it's equal the sliderBeacon.cy)
    //        basicElement1.solvingY: 50
    //==If you need the separate picture for each slider. By default it's the sliderImg
    //        basicElement1.sliderImage.source: ""
    //==The beacon coordinates (if such one exists). 650x50 are default. The y coord should be set as cy if no
    //==        solvingY exists
    //        basicElement1.sliderBeacon.cx: 650
    //        basicElement1.sliderBeacon.cy: 50
    //==If you need the separate picture for each beacon. By default it's the beaconImg
    //        basicElement1.beaconImage.source: beaconImg
    //==If you need a separate drag area for the slider. By default it's common for all of them (minY-maxY)
    //        basicElement1.sliderArea.drag.minimumY: minY
    //        basicElement1.sliderArea.drag.maximumY: maxY

}
