import QtQuick 1.1
import Common 1.0

Item {
    id: puzzleElement

    property alias sliderElement: slider
    property alias sliderImage: sliderImage

    property alias sliderBeacon: beacon
    property alias sliderArea: mouseArea
    property alias beaconImage: beaconImage

    property QtObject moveSound

    property int solvingY: beacon.cy
    property real solvingPrecision: 0.1

    function getMaxValue(first, second) {
        if (first >= second) {
            return first;
        } else {
            return second;
        }
    }

 //============Visual signal (the signal cy is solving cy position for slider by default)==============

    Obj {
        id: beacon
        Image {
            id: beaconImage
        }
        Component.onCompleted: {
            opacity = Math.abs(solvingY - slider.cY) / getMaxValue(mouseArea.drag.maximumY - solvingY, solvingY - mouseArea.drag.minimumY)
        }
    }

//=============Slider==============

    Obj {
        id: slider
        visible: true
        cursor: "Hand"
        Image {
            id: sliderImage
        }
        property int cY: slider.y + slider.height / 2
//        var temp = slider.y
        property int lastY: 0
        property bool solved

        MouseArea {
            id: mouseArea
            anchors.fill: parent
            drag.target: slider
            drag.axis: Drag.YAxis
            drag.minimumY: 0
            drag.maximumY: 400
        }
        Component.onCompleted: {
            lastY = y
            solved = false
//            console.debug("Element",Utils.qmlObjId(parent),"solved:",solved)
        }
        onYChanged: {
            if (!mouseArea.drag.active && visible) {
                if (moveSound) {
                    moveSound.play()
                }
                if (y < mouseArea.drag.minimumY) {
                    y = mouseArea.drag.minimumY
                }
                if (y > mouseArea.drag.maximumY) {
                    y = mouseArea.drag.maximumY
                }
            }
            beacon.opacity = Math.abs(solvingY - cY) / getMaxValue(mouseArea.drag.maximumY - solvingY, solvingY - mouseArea.drag.minimumY)
            if (beacon.opacity < solvingPrecision) {
                if (!slider.solved) {
//                    console.debug("Slider",Utils.qmlObjId(parent),"solved")
//                    console.debug("Slider",Utils.qmlObjId(parent),"cy:",slider.cY)
//                    console.debug("Beacon",Utils.qmlObjId(parent),"cy:",beacon.cy)
//                    console.debug("Beacon opacity="+beacon.opacity)
                    slider.solved = true
                    dragPuzzle.sliderSolved(slider)
                    beacon.opacity = 0
                }
            } else {
//                if (slider.solved) {
//                    console.debug("Slider",Utils.qmlObjId(parent),"unsolved")
//                    console.debug("Slider",Utils.qmlObjId(parent),"cy:",slider.cY)
//                    console.debug("Beacon",Utils.qmlObjId(parent),"cy:",beacon.cy)
//                    console.debug("Beacon opacity="+beacon.opacity)
//                }
                slider.solved = false
            }
        }
    }
}
